@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
@@ -24,7 +23,7 @@
 #include "rectangle.h"
 #include "ucs.h"
 #include "bitmap.h"
-#include "block.h"
+#include "blob.h"
 #include "character.h"
 
 
@@ -32,7 +31,7 @@ Character::Character( const Character & c ) throw()
   : Rectangle( c ), bpv( c.bpv ), gv( c.gv )
   {
   for( unsigned int i = 0; i < bpv.size(); ++i )
-    bpv[i] = new Block( *c.bpv[i] );
+    bpv[i] = new Blob( *c.bpv[i] );
   }
 
 
@@ -44,7 +43,7 @@ Character & Character::operator=( const Character & c ) throw()
     for( unsigned int i = 0; i < bpv.size(); ++i ) delete bpv[i];
     bpv = c.bpv;
     for( unsigned int i = 0; i < bpv.size(); ++i )
-      bpv[i] = new Block( *c.bpv[i] );
+      bpv[i] = new Blob( *c.bpv[i] );
     gv = c.gv;
     }
   return *this;
@@ -57,41 +56,41 @@ Character::~Character() throw()
   }
 
 
-// Return the filled area of the main blocks only (no recursive)
+// Return the filled area of the main blobs only (no recursive)
 //
 int Character::area() const throw()
   {
   int a = 0;
-  for( int i = 0; i < blocks(); ++i ) a += bpv[i]->area();
+  for( int i = 0; i < blobs(); ++i ) a += bpv[i]->area();
   return a;
   }
 
 
-Block & Character::block( int i ) throw()
+Blob & Character::blob( int i ) throw()
   {
-  if( i < 0 || i >= blocks() )
-    Ocrad::internal_error( "block, index out of bounds" );
+  if( i < 0 || i >= blobs() )
+    Ocrad::internal_error( "blob, index out of bounds" );
   return *bpv[i];
   }
 
 
-Block & Character::main_block() throw()
+Blob & Character::main_blob() throw()
   {
   int imax = 0;
-  for( int i = 1; i < blocks(); ++i )
+  for( int i = 1; i < blobs(); ++i )
     if( bpv[i]->size() > bpv[imax]->size() )
       imax = i;
   return *bpv[imax];
   }
 
 
-void Character::shift_blockp( Block * p ) throw()
+void Character::shift_blobp( Blob * p ) throw()
   {
   add_rectangle( *p );
-  int i = blocks() - 1;
+  int i = blobs() - 1;
   for( ; i >= 0; --i )
     {
-    Block & bi = *bpv[i];
+    Blob & bi = *bpv[i];
     if( p->vcenter() > bi.vcenter() ) break;
     if( p->vcenter() == bi.vcenter() && p->hcenter() >= bi.hcenter() ) break;
     }
@@ -168,7 +167,7 @@ bool Character::maybe_letter() const throw()
 
 void Character::join( Character & c ) throw()
   {
-  for( int i = 0; i < c.blocks(); ++i ) shift_blockp( c.bpv[i] );
+  for( int i = 0; i < c.blobs(); ++i ) shift_blobp( c.bpv[i] );
   c.bpv.clear();
   }
 
@@ -235,9 +234,9 @@ void Character::dprint( const Control & control, const Rectangle & charbox,
       bool iscbbot = ( row == charbox.bottom() );
 
       bool ish1top = false, ish1bot = false, ish2top = false, ish2bot = false;
-      if( blocks() == 1 && bpv[0]->holes() )
+      if( blobs() == 1 && bpv[0]->holes() )
         {
-        const Block & b = *bpv[0];
+        const Blob & b = *bpv[0];
         ish1top = ( row == b.hole(0).top() );
         ish1bot = ( row == b.hole(0).bottom() );
         if( b.holes() > 1 )
@@ -250,7 +249,7 @@ void Character::dprint( const Control & control, const Rectangle & charbox,
       for( int col = left(); col <= right(); ++col )
         {
         char ch = ( isvc && col == hcenter() ) ? '+' : '';
-        for( int i = 0; i < blocks(); ++i )
+        for( int i = 0; i < blobs(); ++i )
           {
           int id = bpv[i]->id( row, col );
           if( id != 0 )
