@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 class Character : public Rectangle
@@ -25,7 +24,7 @@ class Character : public Rectangle
     Guess( int c, int v ) throw() : code( c ), value( v ) {}
     };
 
-  std::vector< Block * > bpv;		// the blocks forming this Character
+  std::vector< Blob * > bpv;		// the blobs forming this Character
   std::vector< Guess > gv;		// vector of possible char codes
 					// and their associated values
 
@@ -37,7 +36,7 @@ class Character : public Rectangle
   void recognize13( const Charset & charset, const Rectangle & charbox ) throw();
 
 public:
-  Character( Block * p ) throw()
+  Character( Blob * p ) throw()
     : Rectangle( *p ), bpv( 1, p ) {}
 
   Character( const Rectangle & re, int code, int value ) throw()
@@ -48,11 +47,12 @@ public:
 
   ~Character() throw();
 
-  int   area() const throw();
-  Block & block( int i ) throw();
-  int   blocks() const throw() { return bpv.size(); }
+  int area() const throw();
+  Blob & blob( int i ) throw();
+  int blobs() const throw() { return bpv.size(); }
+  Blob & main_blob() throw();
 
-  void shift_blockp( Block * p ) throw();
+  void shift_blobp( Blob * p ) throw();
 
   void add_guess( int code, int value ) throw();
   void clear_guesses() throw() { gv.clear(); }
@@ -63,6 +63,8 @@ public:
   const Guess & guess( int i ) const throw();
   int guesses() const throw() { return gv.size(); }
   bool maybe( int code ) const throw();
+//  bool maybe_digit() const throw();
+//  bool maybe_letter() const throw();
 
   void join( Character & c ) throw();
   void print( const Control & control ) const throw();
@@ -71,4 +73,5 @@ public:
   void xprint( const Control & control ) const throw();
 
   void recognize1( const Charset & charset, const Rectangle & charbox ) throw();
+  void apply_filter( const Filter & filter ) throw();
   };
