@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <cstdio>
@@ -22,19 +21,19 @@
 #include "rectangle.h"
 #include "ucs.h"
 #include "bitmap.h"
-#include "block.h"
+#include "blob.h"
 #include "character.h"
 #include "profile.h"
 #include "feats.h"
 
 
-// Recognizes 2 block characters.
+// Recognizes 2 blob characters.
 // ij!%:;=?|
 //
 void Character::recognize12( const Charset & charset, const Rectangle & charbox ) throw()
   {
-  const Block & b1 = block( 0 );		// upper block
-  const Block & b2 = block( 1 );		// lower block
+  const Blob & b1 = blob( 0 );		// upper blob
+  const Blob & b2 = blob( 1 );		// lower blob
   int a1 = b1.area();
   int a2 = b2.area();
   Features f1( b1 );
@@ -54,6 +53,7 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
       }
     return;
     }
+
   if( Ocrad::similar( a1, a2, 60 ) )
     {
     if( f1.test_solid( charbox ) == '.' )
@@ -79,19 +79,22 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
           b1.includes_hcenter( b2 ) && 4 * b1.height() > 5 * b2.height() &&
           Ocrad::similar( b1.bottom()-b1.top(), b2.bottom()-b1.top(), 10 ) )
         {
-        Character c2( new Block( b2 ) ); c2.recognize1( charset, charbox );
+        Character c2( new Blob( b2 ) ); c2.recognize1( charset, charbox );
         if( c2.maybe('l') || c2.maybe('|') )
           {
           only_guess( 0, b1.left() ); add_guess( 'f', b2.left() );
           add_guess( 'i', b2.right() ); return;
           }
         }
-      if( 2 * b1.height() > 3 * b2.height() &&
-          b1.holes() == 1 && b2.holes() == 1 &&
-          Ocrad::similar( b2.width(), b2.height(), 50 ) )
-        { add_guess( '%', 0 ); return; }
       }
     }
+
+  if( a1 > a2 && b1.hcenter() < b2.hcenter() &&
+      2 * b1.height() > 3 * b2.height() &&
+      b1.holes() == 1 && b2.holes() == 1 &&
+      Ocrad::similar( b2.width(), b2.height(), 50 ) )
+    { add_guess( '%', 0 ); return; }
+
   if( a1 < a2 )
     {
       {
@@ -110,7 +113,7 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
         if( f2.bp.minima( b2.height() / 4 ) == 2 &&
             b2.top() > b1.bottom() && b2.hcenter() < b1.left() )
           {
-          Character c2( new Block( b2 ) ); c2.recognize1( charset, charbox );
+          Character c2( new Blob( b2 ) ); c2.recognize1( charset, charbox );
           if( c2.maybe('n') )
             {
             if( code == '.' && ( b1.left() < b2.hcenter() || b1.right() > b2.right() ) )
@@ -135,7 +138,7 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
           }
 
         if( code == '.' && f2.bp.minima( b2.height() / 4 ) == 1 &&
-            b1.bottom() <= b2.top() )
+            b1.bottom() <= b2.top() && f2.rp.minima( b2.width() / 2 ) <= 2 )
           {
           int hdiff;
           if( b2.bottom_hook( &hdiff ) && std::abs( hdiff ) >= b2.height() / 2 )
@@ -147,7 +150,8 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
               if( charset.enabled( Charset::iso_8859_15 ) ||
                   charset.enabled( Charset::iso_8859_9 ) )
                 if( -4 * hdiff <= 3 * b2.height() &&
-                    f2.wp.max() > 2 * f1.wp.max() && f2.lp.minima() == 1 )
+                    f2.wp.max() > 2 * f1.wp.max() && f2.lp.minima() == 1 &&
+                    2 * f2.bp[0] < b2.height() )
                   { add_guess( UCS::IQUEST, 0 ); return; }
               add_guess( 'i', 0 ); return;
               }
@@ -161,7 +165,8 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
               if( charset.enabled( Charset::iso_8859_15 ) ||
                   charset.enabled( Charset::iso_8859_9 ) )
                 if( !f2.lp.isctip() && f2.wp.max() >= f1.wp.max() &&
-                    3 * f2.wp[f2.wp.pos(10)] < 2 * f1.wp.max() )
+                    ( 3 * f2.wp[f2.wp.pos(10)] < 2 * f1.wp.max() ||
+                      b1.left() <= b2.left() && b2.vpos( 80 ) > charbox.bottom() ) )
                   { add_guess( UCS::IEXCLAM, 0 ); return; }
               if( maybe_j ) add_guess( 'j', 0 ); else add_guess( 'i', 0 );
               return;
@@ -181,10 +186,10 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
         }
       }
 
-    // upper block has no holes
+    // upper blob has no holes
     if( ( b1.bottom() < b2.vcenter() || 2 * a1 < a2 ) && !b1.holes() )
       {
-      Character c( new Block( b2 ) ); c.recognize1( charset, charbox );
+      Character c( new Blob( b2 ) ); c.recognize1( charset, charbox );
       if( c.guesses() )
         {
         int code = c.guess( 0 ).code;
@@ -193,8 +198,8 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
           int atype = '\'';
           if( UCS::isvowel( code ) && 2 * b1.width() > 3 * b1.height() &&
               !f1.tp.iscpit() && f1.hp.iscpit() ) atype = ':';
-          else if( f1.bp.minima() == 2 ) atype = '^';
-          else if( std::min( b1.height(), b1.width() ) >= 6 &&
+          else if( f1.bp.minima() == 2 || f1.bp.istip() ) atype = '^';
+          else if( std::min( b1.height(), b1.width() ) >= 5 &&
                    ( f1.rp.decreasing() || f1.tp.increasing() ) &&
                    ( f1.bp.decreasing() || f1.lp.increasing() ) ) atype = '`';
           code = UCS::compose( code, atype );
@@ -212,7 +217,7 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
 
   if( b1.bottom() <= b2.top() )
     {
-    int code = f2.test_solid( charbox );
+    const int code = f2.test_solid( charbox );
     if( !b1.holes() && ( code == '.' || 
           ( code && Ocrad::similar( b2.height(), b2.width(), 50 ) ) ) )
       {
