@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
@@ -24,21 +23,21 @@
 #include "rectangle.h"
 #include "ucs.h"
 #include "bitmap.h"
-#include "block.h"
+#include "blob.h"
 #include "character.h"
 #include "profile.h"
 #include "feats.h"
 
 
-// Recognizes 3 block characters.
+// Recognizes 3 blob characters.
 // %
 //
 void Character::recognize13( const Charset & charset, const Rectangle & charbox ) throw()
   {
-  const Block & b1 = block( 0 );
-  const Block & b2 = block( 1 );
-  const Block & b3 = block( 2 );		// lower block
-  Character c( new Block( b3 ) );
+  const Blob & b1 = blob( 0 );
+  const Blob & b2 = blob( 1 );
+  const Blob & b3 = blob( 2 );		// lower blob
+  Character c( new Blob( b3 ) );
   int code = 0;
 
   c.recognize1( charset, charbox );
