@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
@@ -24,7 +23,7 @@
 #include "rectangle.h"
 #include "ucs.h"
 #include "bitmap.h"
-#include "block.h"
+#include "blob.h"
 #include "profile.h"
 #include "feats.h"
 
@@ -37,7 +36,7 @@ int Features::test_235Esz( const Charset & charset ) const throw()
   static const int csize = 3;
   static const int ucoff[csize] = { 0, -1, +1 };
   static const int lcoff[3*csize] = { 0, -1, +1, -1, 0, +1, +1, 0, -1 };
-  const Block & b = *_block;
+  const Blob & b = *_blob;
   if( b.width() < 9 || b.height() > 3 * b.width() ||
       bp.minima( b.height() / 2 ) > 1 ) return 0;
 
@@ -150,7 +149,7 @@ int Features::test_235Esz( const Charset & charset ) const throw()
 
 int Features::test_CEFIJLlT( const Charset & charset ) const throw()
   {
-  const Block & b = *_block;
+  const Blob & b = *_blob;
 
   if( tp.minima( b.height() / 4 ) != 1 || bp.minima( b.height() / 4 ) != 1 )
     return 0;
@@ -265,7 +264,7 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
         return 'l';
       if( b.height() > 3 * wp.max() )
         {
-        if( rp.istip() && lp.ispit() && Ocrad::similar( lp.iminimum(), rp.pos( 50 ), 10 ) )
+        if( rp.istip() && lp.ispit() && Ocrad::similar( lp.iminimum(), lp.pos( 50 ), 10 ) )
           { if( lp.istpit() ) return '{'; else return '('; }
         if( lp.istip() && rp.ispit() && Ocrad::similar( rp.iminimum(), rp.pos( 50 ), 10 ) )
           { if( rp.istpit() ) return '}'; else return ')'; }
@@ -306,7 +305,7 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
       if( 2 * b.height() >= 3 * b.width() &&
           vbar(0).height() >= 3 * vbar(0).width() &&
           lp.istpit() && lp.minima() == 1 )
-        { int i = lp.iminimum();
+        { const int i = lp.iminimum();
           if( i > lp.pos( 10 ) && i < lp.pos( 40 ) ) return '1'; }
       }
     }
@@ -320,7 +319,7 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
 
 int Features::test_c() const throw()
   {
-  const Block & b = *_block;
+  const Blob & b = *_blob;
 
   if( lp.isconvex() || lp.ispit() )
     {
@@ -354,7 +353,7 @@ int Features::test_c() const throw()
 
 int Features::test_frst( const Rectangle & charbox ) const throw()
   {
-  const Block & b = *_block;
+  const Blob & b = *_blob;
 
   if( bp.minima( b.height() / 4 ) != 1 || tp.minima( b.height() / 2 ) != 1 ||
       bp.minima( b.height() / 2 ) != 1 ) return 0;
@@ -469,7 +468,7 @@ int Features::test_frst( const Rectangle & charbox ) const throw()
 
 int Features::test_G() const throw()
   {
-  const Block & b = *_block;
+  const Blob & b = *_blob;
 
   if( lp.isconvex() || lp.ispit() )
     {
@@ -508,14 +507,16 @@ int Features::test_G() const throw()
 //
 int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const throw()
   {
-  const Block & b = *_block;
+  const Blob & b = *_blob;
 
   if( tp.minima( b.height() / 5 ) == 2 && tp.minima( b.height() / 4 ) == 2 &&
       tp.isctip() )
     {
     const int noise = ( std::min( b.height(), b.width() ) / 30 ) + 1;
+    const int m5 = bp.minima( b.height() / 5 );
     if( 2 * b.height() >= b.width() && b.height() >= 10 &&
-        bp.minima( b.height() / 5 ) == 1 )
+        ( m5 == 1 ||
+          ( m5 == 2 && Ocrad::similar( bp.iminimum(), bp.pos( 50 ), 10 ) ) ) )
       {
       const int lg = lp.min( lp.pos( 90 ) );
       if( lg > 1 && bp.isvpit() && tp.minima( b.height() / 2 ) == 2 &&
@@ -610,7 +611,7 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const throw()
 //
 int Features::test_hknwx( const Rectangle & charbox ) const throw()
   {
-  const Block & b = *_block;
+  const Blob & b = *_blob;
   const int m8 = tp.minima( b.height() / 8 );
 
   if( m8 == 2 && bp.minima( b.height() / 2 ) == 1 &&
@@ -657,7 +658,7 @@ int Features::test_hknwx( const Rectangle & charbox ) const throw()
 //
 int Features::test_s_cedilla() const throw()
   {
-  const Block & b = *_block;
+  const Blob & b = *_blob;
   int urow2 = 0, urow3 = 0, urow4 = 0, col, black_section = 0;
 
   for( col = b.hcenter() - 1; col <= b.hcenter() + 1; ++col )
@@ -688,7 +689,7 @@ int Features::test_s_cedilla() const throw()
 
 bool Features::test_comma() const throw()
   {
-  const Block & b = *_block;
+  const Blob & b = *_blob;
 
   if( b.holes() || b.height() <= b.width() || b.height() > 3 * b.width() )
     return false;
@@ -712,7 +713,7 @@ bool Features::test_comma() const throw()
 
 int Features::test_easy( const Rectangle & charbox ) const throw()
   {
-  const Block & b = *_block;
+  const Blob & b = *_blob;
   int code = test_solid( charbox );
   if( code ) return code;
 
@@ -734,7 +735,7 @@ int Features::test_easy( const Rectangle & charbox ) const throw()
 //
 int Features::test_line( const Rectangle & charbox ) const throw()
   {
-  const Block & b = *_block;
+  const Blob & b = *_blob;
   int slope1, slope2;
 
   if( tp.minima() != 1 ) return 0;
@@ -798,7 +799,7 @@ int Features::test_line( const Rectangle & charbox ) const throw()
 
 int Features::test_solid( const Rectangle & charbox ) const throw()
   {
-  const Block & b = *_block;
+  const Blob & b = *_blob;
   if( b.holes() ) return 0;
 
   if( b.height() >= 5 && b.width() >= 5 )
