@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
@@ -24,7 +23,7 @@
 #include "rectangle.h"
 #include "ucs.h"
 #include "bitmap.h"
-#include "block.h"
+#include "blob.h"
 #include "profile.h"
 #include "feats.h"
 
@@ -34,7 +33,7 @@
 //
 int Features::test_49ARegpq( const Rectangle & charbox ) const throw()
   {
-  const Block & b = *_block;
+  const Blob & b = *_blob;
   const Bitmap & h = b.hole( 0 );
 
   if( bp.minima( b.height() / 10 + 1 ) == 2 && bp.isctip() && tp.minima() == 1 )
@@ -98,7 +97,7 @@ int Features::test_49ARegpq( const Rectangle & charbox ) const throw()
 
 int Features::test_4ADQao( const Charset & charset, const Rectangle & charbox ) const throw()
   {
-  const Block & b = *_block;
+  const Blob & b = *_blob;
   const Bitmap & h = b.hole( 0 );
   int left_delta = h.left() - b.left(), right_delta = b.right() - h.right();
   const int noise = ( std::min( b.height(), b.width() ) / 30 ) + 1;
@@ -165,7 +164,7 @@ int Features::test_4ADQao( const Charset & charset, const Rectangle & charbox )
 //
 int Features::test_6abd( const Charset & charset ) const throw()
   {
-  const Block & b = *_block;
+  const Blob & b = *_blob;
   const Bitmap & h = b.hole( 0 );
 
   if( 3 * h.width() < b.width() &&
@@ -212,8 +211,9 @@ int Features::test_6abd( const Charset & charset ) const throw()
 
   if( b.escape_left( row, col ) )
     {
-    int row2 = b.seek_top( h.top(), h.left(), false ) - 1;
-    row2 = b.seek_top( row2, h.left() );
+    const int col2 = std::max( h.left(), h.hpos( 10 ) );
+    int row2 = b.seek_top( h.top(), col2, false ) - 1;
+    row2 = b.seek_top( row2, col2 );
     if( row2 > b.top() )
       {
       if( charset.enabled( Charset::iso_8859_15 ) ||
