@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 /*
     Return values: 0 for a normal exit, 1 for environmental problems
@@ -39,25 +38,21 @@
 
 namespace {
 
-// Date of this version: 2006-04-03
-
 const char * invocation_name = 0;
 const char * const Program_name    = "GNU Ocrad";
 const char * const program_name    = "ocrad";
-const char * const program_version = "0.15";
-const char * const program_year    = "2006";
+const char * const program_year    = "2007";
 
 struct Input_control
   {
   Transformation transformation;
   int rindex, layout_level, scale;
   Rational threshold, ltrb[4];
-  char specialtype;
   bool copy, crop, invert;
 
   Input_control() throw()
     : rindex( -1 ), layout_level( 0 ), scale( 0 ), threshold( -1 ),
-      specialtype( 0 ), copy( false ), crop( false ), invert( false ) {}
+      copy( false ), crop( false ), invert( false ) {}
 
   bool read_crop_rectangle( const char * ptr ) throw();
   bool set_threshold( const char * ptr ) throw();
@@ -77,6 +72,14 @@ void show_error( const char * msg, const int errcode = 0, const bool help = fals
   }
 
 
+bool maybe_less( const Rational a, const Rational b ) throw()
+  {
+  if( a >= 0 && a <= 1 && b >= 0 && b <= 1 ) return a < b;
+  if( a > 1 && b > 1 ) return a < b;
+  return ( a >= 0 && b > 0 );
+  }
+
+
 bool Input_control::read_crop_rectangle( const char * ptr ) throw()
   {
   int c = ltrb[0].parse( ptr );				// left
@@ -87,10 +90,10 @@ bool Input_control::read_crop_rectangle( const char * ptr ) throw()
     if( c && ltrb[1] >= 0 && ptr[i+c] == ',' )
       {
       i += c + 1; c = ltrb[2].parse( &ptr[i] );		// right
-      if( c && ltrb[2] > 0 && ptr[i+c] == ',' )
+      if( c && maybe_less( ltrb[0], ltrb[2] ) && ptr[i+c] == ',' )
         {
         i += c + 1; c = ltrb[3].parse( &ptr[i] );	// bottom
-        if( c && ltrb[3] > 0 ) { crop = true; return true; }
+        if( c && maybe_less( ltrb[1], ltrb[3] ) ) { crop = true; return true; }
         }
       }
     }
@@ -120,6 +123,7 @@ void show_help() throw()
   std::printf( "  -a, --append             append text to output file\n" );
   std::printf( "  -b, --block=<n>          process only the specified text block\n" );
   std::printf( "  -c, --charset=<name>     try `--charset=help' for a list of names\n" );
+  std::printf( "  -e, --filter=<name>      try `--filter=help' for a list of names\n" );
   std::printf( "  -f, --force              force overwrite of output file\n" );
   std::printf( "  -F, --format=<fmt>       output format (byte, utf8)\n" );
   std::printf( "  -i, --invert             invert image levels (white on black)\n" );
@@ -136,7 +140,6 @@ void show_help() throw()
     std::printf( "  -1..6                    pnm output file type (debug)\n" );
     std::printf( "  -C, --copy               'copy' input to output (debug)\n" );
     std::printf( "  -D, --debug=<level>      (0-100) output intermediate data (debug)\n" );
-    std::printf( "  -S <type>                make a 'special file' (debug)\n" );
     }
   std::printf( "\nReport bugs to bug-ocrad@gnu.org\n" );
   }
@@ -144,10 +147,11 @@ void show_help() throw()
 
 void show_version() throw()
   {
-  std::printf( "%s version %s\n", Program_name, program_version );
+  std::printf( "%s %s\n", Program_name, PROGVERSION );
   std::printf( "Copyright (C) %s Antonio Diaz Diaz.\n", program_year );
-  std::printf( "This program is free software; you may redistribute it under the terms of\n" );
-  std::printf( "the GNU General Public License.  This program has absolutely no warranty.\n" );
+  std::printf( "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n" );
+  std::printf( "This is free software: you are free to change and redistribute it.\n" );
+  std::printf( "There is NO WARRANTY, to the extent permitted by law.\n" );
   }
 
 
@@ -188,17 +192,6 @@ int process_file( FILE *infile, const char * infile_name,
       std::fprintf( stderr,"This page has only %d text block(s)\n", page_image.zones() );
       return 1;
       }
-    if( input_control.specialtype != 0 )
-      {
-      if( control.outfile )
-        {
-        const char & t = input_control.specialtype;
-        if( t == 'v' || t == 'h' ) page_image.histogramize( t == 'v' );
-        else { show_error( "bad special type" ); return 1; }
-        page_image.save( control.outfile, control.filetype );
-        }
-      return 0;
-      }
     if( input_control.copy )
       {
       if( control.outfile )
@@ -239,7 +232,7 @@ void Ocrad::internal_error( const char * msg ) throw()
 // image. (or for a pnm file if debugging).
 // 'exportfile' is the Ocr Results File.
 //
-int main( int argc, char * argv[] ) throw()
+int main( const int argc, const char * argv[] ) throw()
   {
   Input_control input_control;
   Control control;
@@ -247,7 +240,7 @@ int main( int argc, char * argv[] ) throw()
   bool append = false, force = false;
   invocation_name = argv[0];
 
-  static const Arg_parser::Option options[] =
+  const Arg_parser::Option options[] =
     {
     { '1', 0,           Arg_parser::no  },
     { '2', 0,           Arg_parser::no  },
@@ -260,6 +253,7 @@ int main( int argc, char * argv[] ) throw()
     { 'c', "charset",   Arg_parser::yes },
     { 'C', "copy",      Arg_parser::no  },
     { 'D', "debug",     Arg_parser::yes },
+    { 'e', "filter",    Arg_parser::yes },
     { 'f', "force",     Arg_parser::no  },
     { 'F', "format",    Arg_parser::yes },
     { 'h', "help",      Arg_parser::no  },
@@ -268,7 +262,6 @@ int main( int argc, char * argv[] ) throw()
     { 'o', 0,           Arg_parser::yes },
     { 'p', "crop",      Arg_parser::yes },
     { 's', "scale",     Arg_parser::yes },
-    { 'S', 0,           Arg_parser::yes },
     { 't', "transform", Arg_parser::yes },
     { 'T', "threshold", Arg_parser::yes },
     { 'v', "verbose",   Arg_parser::no  },
@@ -299,7 +292,6 @@ int main( int argc, char * argv[] ) throw()
       case 'F': if( !control.set_format( arg ) )
                   { show_error( "bad output format", 0, true ); return 1; }
                 break;
-      case 'S': input_control.specialtype = arg[0]; break;
       case 'T': if( !input_control.set_threshold( arg ) ) return 1; break;
       case 'V':	show_version(); return 0;
       case 'a': append = true; break;
@@ -307,6 +299,9 @@ int main( int argc, char * argv[] ) throw()
       case 'c': if( !control.charset.enable( arg ) )
                   { control.charset.show_error( program_name, arg ); return 1; }
                 break;
+      case 'e': if( !control.filter.set( arg ) )
+                  { control.filter.show_error( program_name, arg ); return 1; }
+                break;
       case 'f': force = true; break;
       case 'h': show_help(); return 0;
       case 'i': input_control.invert = true; break;
@@ -337,9 +332,7 @@ int main( int argc, char * argv[] ) throw()
       { std::fprintf( stderr, "Cannot open %s\n", outfile_name ); return 1; }
     }
 
-  if( exportfile_name && control.debug_level == 0 &&
-      input_control.scale == 0 && input_control.specialtype == 0 &&
-      !input_control.copy )
+  if( exportfile_name && control.debug_level == 0 && !input_control.copy )
     {
     if( std::strcmp( exportfile_name, "-" ) == 0 )
       { control.exportfile = stdout; if( !outfile_name ) control.outfile = 0; }
@@ -354,7 +347,7 @@ int main( int argc, char * argv[] ) throw()
       }
     std::fprintf( control.exportfile,
                   "# Ocr Results File. Created by %s version %s\n",
-                  Program_name, program_version );
+                  Program_name, PROGVERSION );
     }
 
   // process any remaining command line arguments (input files)
@@ -369,7 +362,7 @@ int main( int argc, char * argv[] ) throw()
       if( argind >= parser.arguments() ) { infile = 0; break; }
       infile_name = parser.argument( argind++ ).c_str();
       if( std::strcmp( infile_name, "-" ) == 0 ) infile = stdin;
-      else infile = std::fopen( infile_name, "r" );
+      else infile = std::fopen( infile_name, "rb" );
       if( infile ) break;
       std::fprintf( stderr, "Cannot open %s\n", infile_name );
       if( retval == 0 ) retval = 1;
