@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
@@ -146,7 +145,7 @@ void Page_image::find_rows( const Rectangle & rin, bool recursive ) throw()
 
     if( b < ibottom )			// cut by a minimum near the center
       {
-      int mid = ( b + t ) / 2, half = ( b - t ) / 2;
+      const int mid = ( b + t ) / 2, half = ( b - t ) / 2;
       b = mid;
       for( int i = 1; i <= half && v_outline[b] > 0; ++i )
         {
@@ -246,11 +245,11 @@ int Page_image::analyse_layout( const int layout_level ) throw()
   if( layout_level >= 1 && layout_level <= 2 &&
       left() == 0 && top() == 0 && width() > 200 && height() > 200 )
     {
-    Page_image reduced( *this, 10, Rational( 9, 100 ) );
+    Page_image reduced( *this, 8, Rational( 7, 64 ) );
     reduced.find_columns( reduced, layout_level >= 2 );
     rv = reduced.rv;
     for( unsigned int i = 0; i < rv.size(); ++i )
-      rv[i].enlarge( 10 );
+      rv[i].enlarge( 8 );
     }
   if( rv.size() == 0 ) rv.push_back( *this );
   tv.clear(); tv.insert( tv.end(), rv.size(), _threshold );
@@ -293,22 +292,3 @@ void Page_image::draw_track( const Track & tr ) throw()
     if( row >= top() && row <= bottom() ) set_bit( row, col, true );
     }
   }
-
-
-void Page_image::histogramize( const bool vertical ) throw()
-  {
-  if( vertical )
-    {
-    for( int col = left(); col <= right(); ++col )
-      for( int row = top(), y = bottom(); row <= bottom(); ++row )
-        if( get_bit( row, col ) )
-          { set_bit( row, col, false ); set_bit( y--, col, true ); }
-    }
-  else
-    {
-    for( int row = top(); row <= bottom(); ++row )
-      for( int col = left(), x = left(); col <= right(); ++col )
-        if( get_bit( row, col ) )
-          { set_bit( row, col, false ); set_bit( row, x++, true ); }
-    }
-  }
