@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
@@ -23,7 +22,7 @@
 #include "common.h"
 #include "rectangle.h"
 #include "bitmap.h"
-#include "block.h"
+#include "blob.h"
 #include "profile.h"
 
 
@@ -173,7 +172,7 @@ int Profile::area( int l, int r ) throw()
 bool Profile::increasing( int i ) throw()
   {
   if( _limit < 0 ) initialize();
-  if( i <= 0 || i > samples() - 2 || data[samples()-1] - data[i] < 2 )
+  if( i < 0 || i > samples() - 2 || data[samples()-1] - data[i] < 2 )
     return false;
   while( ++i < samples() ) if( data[i] < data[i-1] ) return false;
   return true;
@@ -185,7 +184,7 @@ bool Profile::decreasing( int i ) throw()
   if( _limit < 0 ) initialize();
   const int noise = ( std::min( samples(), _limit ) / 20 ) + 1;
   if( i < 0 || samples() - i <= 2 * noise ||
-      data[samples()-noise] - data[i] > -( noise + 1 ) )
+      data[i] - data[samples()-noise] < noise + 1 )
     return false;
   while( ++i < samples() - noise ) if( data[i] > data[i-1] ) return false;
   return true;
@@ -349,10 +348,12 @@ bool Profile::iscpit( const int cpos ) throw()
   {
   if( _limit < 0 ) initialize();
   if( samples() < 5 || cpos < 25 || cpos > 75 ) return false;
-  int th = ( mean() < 2 ) ? 2 : mean();
-  int imin = -1, mid = ( ( samples() - 1 ) * cpos ) / 100;
+  const int mid = ( ( samples() - 1 ) * cpos ) / 100;
+  const int iend = std::min( samples() / 4, std::min( mid, samples() - mid ) );
+  const int th = ( ( mean() < 2 ) ? 2 : mean() );
+  int imin = -1;
 
-  for( int i = 0; i < samples() / 4; ++i )
+  for( int i = 0; i < iend; ++i )
     {
     if( data[mid+i] < th ) { imin = mid + i; break; }
     if( data[mid-i-1] < th ) { imin = mid - i - 1; break; }
@@ -507,10 +508,11 @@ bool Profile::isctip( const int cpos ) throw()
   if( _limit < 0 ) initialize();
   if( samples() < 5 || cpos < 25 || cpos > 75 ) return false;
   const int mid = ( ( samples() - 1 ) * cpos ) / 100;
+  const int iend = std::min( samples() / 4, std::min( mid, samples() - mid ) );
   int th = std::max( 2, std::min( mean(), _limit / 3 ) );
   int imax = -1;
 
-  for( int i = 0; i < samples() / 4; ++i )
+  for( int i = 0; i < iend; ++i )
     {
     if( data[mid+i] > th ) { imax = mid + i; break; }
     if( data[mid-i-1] > th ) { imax = mid - i - 1; break; }
@@ -518,7 +520,7 @@ bool Profile::isctip( const int cpos ) throw()
   if( imax < 0 && mean() == 0 )
     {
     --th;
-    for( int i = 0; i < samples() / 4; ++i )
+    for( int i = 0; i < iend; ++i )
       {
       if( data[mid+i] > th ) { imax = mid + i; break; }
       if( data[mid-i-1] > th ) { imax = mid - i - 1; break; }
@@ -526,6 +528,7 @@ bool Profile::isctip( const int cpos ) throw()
     }
   if( imax < 0 ) return false;
 
+  th = std::max( th, data[imax] / 2 );
   for( int i = imax + 1; i < samples(); ++i )
     if( data[i] < th )
       {
