@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,14 +12,14 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <algorithm>
 #include <cctype>
 #include <climits>
-#include <cstdio>
 #include <cstdlib>
+#include <string>
 
 #include "rational.h"
 
@@ -120,7 +120,7 @@ bool Rational::operator<( const Rational & r ) const throw()
 
 Rational Rational::inverse() const throw()
   {
-  Rational tmp = den;
+  Rational tmp( den );
   if( num == 0 ) { tmp.num = INT_MAX; tmp.den = 1; }
   else if( num < 0 ) { tmp.num = -den; tmp.den = -num; }
   else tmp.den = num;
@@ -196,27 +196,30 @@ int Rational::parse( const char * ptr ) throw()
 // Returns the fraction "num/den" as a floating point with "prec" decimals.
 // If 'prec' is negative, only the needed decimals are shown.
 //
-const char * Rational::to_decimal( const int iwidth, int prec ) const throw()
+const std::string Rational::to_decimal( const int iwidth, int prec ) const throw()
   {
-  static const int bufsize = 80;
-  static char buf[bufsize+1];
+  std::string s;
+
   if( den == 0 )
     {
-    if( num == 0 ) return "NAN";
-    else if( num > 0 ) return "+INF"; else return "-INF";
+    if( num == 0 ) s = "NAN"; else if( num > 0 ) s = "+INF"; else s = "-INF";
+    return s;
     }
-  int index = 0;
-  bool trunc = false;
-  if( prec < 0 ) { trunc = true; prec = -prec; }
+  bool negative = false, trunc = false;
+  int ipart = num / den;
+  if( ipart < 0 ) { ipart = -ipart; negative = true; }
+  if( prec < 0 ) { prec = -prec; trunc = true; }
 
-  index += std::snprintf( buf, bufsize, "%*d", iwidth, num / den );
+  do { s += '0' + ( ipart % 10 ); ipart /= 10; } while( ipart > 0 );
+  if( negative ) s += '-';
+  if( iwidth > (int)s.size() ) s.append( iwidth - s.size(), ' ' );
+  std::reverse( s.begin(), s.end() );
   long long rest = std::abs( num ) % den;
-  if( index < bufsize - 1 && prec > 0 && ( rest > 0 || !trunc ) )
+  if( prec > 0 && ( rest > 0 || !trunc ) )
     {
-    buf[index++] = '.';
-    while( index < bufsize && prec > 0 && ( rest > 0 || !trunc ) )
-      { rest *= 10; buf[index++] = '0' + ( rest / den ); rest %= den; --prec; }
-    buf[index] = 0;
+    s += '.';
+    while( prec > 0 && ( rest > 0 || !trunc ) )
+      { rest *= 10; s += '0' + ( rest / den ); rest %= den; --prec; }
     }
-  return buf;
+  return s;
   }
