@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,14 +12,14 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
 #include <cstddef>
 #include <cstdio>
 #include <cstdlib>
+
 #include "common.h"
 #include "rectangle.h"
 
@@ -193,7 +193,7 @@ bool Rectangle::v_overlaps( const Rectangle & re ) const throw()
   { return ( _top <= re._bottom && _bottom >= re._top ); }
 
 
-bool Rectangle::is_hcentered_in( const Rectangle & re ) const throw()
+bool Rectangle::is_hcentred_in( const Rectangle & re ) const throw()
   {
   if( this->h_includes( re.hcenter() ) ) return true;
   int w = std::min( re.height(), re.width() ) / 2;
@@ -207,7 +207,7 @@ bool Rectangle::is_hcentered_in( const Rectangle & re ) const throw()
   }
 
 
-bool Rectangle::is_vcentered_in( const Rectangle & re ) const throw()
+bool Rectangle::is_vcentred_in( const Rectangle & re ) const throw()
   {
   if( this->v_includes( re.vcenter() ) ) return true;
   int h = std::min( re.height(), re.width() ) / 2;
