@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
@@ -25,7 +24,7 @@
 #include "rectangle.h"
 #include "track.h"
 #include "bitmap.h"
-#include "block.h"
+#include "blob.h"
 #include "character.h"
 #include "page_image.h"
 #include "textline.h"
@@ -69,13 +68,13 @@ void join_characters( std::vector< Textline * > & tlpv ) throw()
         else cup = &c2, cdn = &c1;
         if( cdn->includes_hcenter( *cup ) || cup->includes_hcenter( *cdn ) ||
             ( cdn->top() > cup->bottom() && cdn->hcenter() < cup->hcenter() ) ||
-            ( cdn->blocks() == 2 &&
-              2 * cdn->block( 0 ).size() < cdn->block( 1 ).size() &&
-              cdn->block( 0 ).includes_vcenter( *cup ) ) )
+            ( cdn->blobs() == 2 &&
+              2 * cdn->blob( 0 ).size() < cdn->blob( 1 ).size() &&
+              cdn->blob( 0 ).includes_vcenter( *cup ) ) )
           {
           int k;
-          if( 64 * c1.size() < c2.main_block().size() ) k = i;
-          else if( 64 * c2.size() < c1.main_block().size() ) k = j;
+          if( 64 * c1.size() < c2.main_blob().size() ) k = i;
+          else if( 64 * c2.size() < c1.main_blob().size() ) k = j;
           else if( cdn == &c2 ) { c2.join( c1 ); k = i; }
           else { c1.join( c2 ); k = j; }
           line.delete_character( k );
@@ -87,107 +86,42 @@ void join_characters( std::vector< Textline * > & tlpv ) throw()
     }
   }
 
-
-// Return the character position >= first preceding a big gap or eol.
-//
-int find_big_gap( const Textline & line, const int first, const int space_width_limit ) throw()
-  {
-  int i = first;
-  while( i + 1 < line.characters() )
-    {
-    Character & c1 = line.character( i );
-    Character & c2 = line.character( i + 1 );
-    const int gap = c2.left() - c1.right() - 1;
-    if( gap > space_width_limit ) break; else ++i;
-    }
-  return i;
-  }
-
-
-// Insert spaces between characters.
-//
-void insert_spaces( std::vector< Textline * > & tlpv ) throw()
-  {
-  for( unsigned int current_line = 0; current_line < tlpv.size(); ++current_line )
-    {
-    Textline & line1 = *tlpv[current_line];
-    const Rational mw = line1.mean_width();
-    if( mw < 2 ) continue;
-    const int space_width_limit = ( 3 * mw ).trunc();
-    int first = 0;
-    while( first + 1 < line1.characters() )
-      {
-      int last = find_big_gap( line1, first, space_width_limit );
-      const Rational mg = line1.mean_gap_width( first, last );
-      if( first < last && mg >= 0 )
-        {
-        int spaces = 0, nospaces = 0, spsum = 0, nospsum = 0;
-        for( int i = first ; i < last; ++i )
-          {
-          Character & c1 = line1.character( i );
-          Character & c2 = line1.character( i + 1 );
-          const int gap = c2.left() - c1.right() - 1;
-          if( gap >= mw.trunc() || gap > 3 * mg ||
-              ( 5 * gap > 2 * mw && gap > 2 * mg ) ||
-              ( 3 * c1.width() > 2 * mw && 3 * c2.width() > 2 * mw && 2 * gap > mw && 5 * gap > 8 * mg ) )
-            { ++spaces; spsum += gap;
-              if( line1.insert_space( i + 1 ) ) { ++i; ++last; } }
-          else { ++nospaces; nospsum += gap; }
-          }
-        if( spaces && nospaces )
-          {
-          const Rational th = ( Rational( 3 * spsum, spaces ) + Rational( nospsum, nospaces ) ) / 4;
-          for( int i = first ; i < last; ++i )
-            {
-            Character & c1 = line1.character( i );
-            Character & c2 = line1.character( i + 1 );
-            const int gap = c2.left() - c1.right() - 1;
-            if( gap > th && line1.insert_space( i + 1 ) ) { ++i; ++last; }
-            }
-          }
-        }
-      if( ++last < line1.characters() && line1.insert_space( last, true ) )
-        ++last;
-      first = last;
-      }
-    }
-  }
-
 } // end namespace
 
 
 Textblock::Textblock( const Rectangle & r,
-                      std::vector< std::vector< Block * > > & blockp_matrix ) throw()
+                      std::vector< std::vector< Blob * > > & blobp_matrix ) throw()
   : Rectangle( r )
   {
-  int cuts = blockp_matrix.size();
+  int cuts = blobp_matrix.size();
   std::vector< int > mean_height( cuts, 0 );
-  std::vector< std::vector< Block * > > pending( cuts );
-  std::vector< std::vector< Block * > > pending_tall( cuts );
-  std::vector< std::vector< Block * > > pending_short( cuts );
+  std::vector< std::vector< Blob * > > pending( cuts );
+  std::vector< std::vector< Blob * > > pending_tall( cuts );
+  std::vector< std::vector< Blob * > > pending_short( cuts );
 
-  // Classify blocks by height.
+  // Classify blobs by height.
   for( int cut = 0; cut < cuts; ++cut )
     {
-    const std::vector< Block * > & blockp_vector = blockp_matrix[cut];
-    if( !blockp_vector.size() ) continue;
+    const std::vector< Blob * > & blobp_vector = blobp_matrix[cut];
+    if( !blobp_vector.size() ) continue;
     unsigned int samples = 0;
     std::vector< int > height_distrib;
-    for( unsigned int i = 0; i < blockp_vector.size(); ++i )
+    for( unsigned int i = 0; i < blobp_vector.size(); ++i )
       {
-      unsigned int h = blockp_vector[i]->height();
-      unsigned int w = blockp_vector[i]->width();
+      unsigned int h = blobp_vector[i]->height();
+      unsigned int w = blobp_vector[i]->width();
       if( h < 10 || w >= 3 * h ) continue;
       if( h >= height_distrib.size() ) height_distrib.resize( h + 1 );
       ++height_distrib[h]; ++samples;
       }
     if( !height_distrib.size() )
-      for( unsigned int i = 0; i < blockp_vector.size(); ++i )
+      for( unsigned int i = 0; i < blobp_vector.size(); ++i )
         {
-        unsigned int h = blockp_vector[i]->height();
+        unsigned int h = blobp_vector[i]->height();
         if( h >= height_distrib.size() ) height_distrib.resize( h + 1 );
         ++height_distrib[h]; ++samples;
         }
+
     int valid_samples = 0;
     for( unsigned int i = 0, count = 0; i < height_distrib.size(); ++i )
       {
@@ -198,9 +132,9 @@ Textblock::Textblock( const Rectangle & r,
       }
     if( valid_samples ) mean_height[cut] /= valid_samples;
 
-    for( unsigned int i = 0; i < blockp_vector.size(); ++i )
+    for( unsigned int i = 0; i < blobp_vector.size(); ++i )
       {
-      Block * p = blockp_vector[i];
+      Blob * p = blobp_vector[i];
       if( p->height() >= 2 * mean_height[cut] )
         pending_tall[cut].push_back( p );
       else if( 2 * p->height() <= mean_height[cut] )
@@ -209,7 +143,7 @@ Textblock::Textblock( const Rectangle & r,
       }
     }
 
-  // Assign normal blocks to characters and create lines.
+  // Assign normal blobs to characters and create lines.
   int min_line = 0;	// first line of current cut
   add_line( tlpv );
   for( int cut = 0; cut < cuts; ++cut )
@@ -221,7 +155,7 @@ Textblock::Textblock( const Rectangle & r,
       tlpv[current_line]->shift_characterp( new Character( pending[cut][0] ) );
       for( unsigned int i = 1; i < pending[cut].size(); ++i )
         {
-        Block & b = *pending[cut][i];
+        Blob & b = *pending[cut][i];
         current_line = std::max( min_line, current_line - 2 );
         while( true )
           {
@@ -259,13 +193,13 @@ Textblock::Textblock( const Rectangle & r,
   // Create tracks of lines.
   for( int i = 0; i < textlines(); ++i ) tlpv[i]->set_track();
 
-  // Insert tall blocks.
+  // Insert tall blobs.
   // Seek up, then seek down, needed for slanted or curved lines.
   for( int current_line = 0, cut = 0; cut < cuts; ++cut )
     {
     for( unsigned int i = 0; i < pending_tall[cut].size(); ++i )
       {
-      Block & b = *pending_tall[cut][i];
+      Blob & b = *pending_tall[cut][i];
       while( current_line > 0 &&
              b.bottom() < tlpv[current_line]->vcenter( b.hcenter() ) )
         --current_line;
@@ -275,36 +209,32 @@ Textblock::Textblock( const Rectangle & r,
       if( current_line >= textlines() )
         { --current_line; delete &b; continue; }
       Textline & l = *tlpv[current_line];
-      if( b.height() <= 3 * l.mean_height() &&
-          ( b.height() <= 2 * l.mean_height() ||
-            ( l.big_initial() && l.big_initial()->left() < b.left() ) ||
-            ( !l.big_initial() && l.character(0).left() < b.left() ) ) )
+      const int bi = l.big_initials();
+      const int mh = l.mean_height();
+      if( b.height() <= 3 * mh &&
+          ( b.height() <= 2 * mh || l.character( bi ).left() < b.left() ) )
         l.shift_characterp( new Character( &b ) );
       else		// possible big initial
         {
-        if( l.big_initial() && l.big_initial()->height() <= 2 * l.mean_height() )
-          {
-          l.shift_characterp( new Character( *l.big_initial() ) );
-          l.big_initial( 0 );
-          }
-        if( ( !l.big_initial() || l.big_initial()->left() > b.left() ) &&
-            l.character(0).left() > b.hcenter() )
-          l.big_initial( new Character( &b ) );
+        if( l.character( std::min( bi+1, l.characters() - 1 ) ).left() > b.hcenter() )
+          l.shift_characterp( new Character( &b ), true );
         }
       }
     }
 
-  // Insert short blocks.
+//  for( int i = 0; i < textlines(); ++i ) tlpv[i]->verify_big_initials();
+
+  // Insert short blobs.
   // Seek up, then seek down, needed for slanted or curved lines.
   for( int current_line = 0, cut = 0; cut < cuts; ++cut )
     {
     for( unsigned int i = 0; i < pending_short[cut].size(); ++i )
       {
-      Block & b = *pending_short[cut][i];
+      Blob & b = *pending_short[cut][i];
       while( current_line > 0 &&
              b.bottom() < tlpv[current_line]->top( b.hcenter() ) )
         --current_line;
-      int temp = current_line;
+      int temp = std::max( 0, current_line - 1 );
       while( current_line < textlines() &&
              b.top() > tlpv[current_line]->bottom( b.hcenter() ) )
         ++current_line;
@@ -370,6 +300,8 @@ Textblock::Textblock( const Rectangle & r,
 
   join_characters( tlpv );
 
+  for( int i = 0; i < textlines(); ++i ) tlpv[i]->verify_big_initials();
+
   // Fourth pass. Remove noise lines.
   if( textlines() >= 3 )
     {
@@ -390,12 +322,12 @@ Textblock::Textblock( const Rectangle & r,
   for( int i = 0; i < textlines(); ++i )
     {
     Textline & l = *tlpv[i];
-    if( !l.big_initial() && l.characters() > 2 )
+    if( !l.big_initials() && l.characters() > 2 )
       {
       const Character & c0 = l.character( 0 );
       const Character & c1 = l.character( 1 );
       const Character & c2 = l.character( 2 );
-      if( c0.blocks() == 1 &&
+      if( c0.blobs() == 1 &&
           4 * c0.size() < c1.size() && c1.left() - c0.right() > 2 * l.height() &&
           4 * c0.size() < c2.size() && c2.left() - c1.right() < l.height() )
         l.delete_character( 0 );
@@ -405,13 +337,13 @@ Textblock::Textblock( const Rectangle & r,
       const Character & c0 = l.character( l.characters() - 1 );
       const Character & c1 = l.character( l.characters() - 2 );
       const Character & c2 = l.character( l.characters() - 3 );
-      if( c0.blocks() == 1 &&
+      if( c0.blobs() == 1 &&
           4 * c0.size() < c1.size() && c0.left() - c1.right() > 2 * l.height() &&
           4 * c0.size() < c2.size() && c1.left() - c2.right() < l.height() )
         l.delete_character( l.characters() - 1 );
       }
     }
-  insert_spaces( tlpv );
+  for( int i = 0; i < textlines(); ++i ) tlpv[i]->insert_spaces();
   }
 
 
@@ -441,10 +373,8 @@ void Textblock::recognize( const Charset & charset, const Filter & filter ) thro
     {
     Textline & line1 = *tlpv[i];
     bool flag = false;
-    if( line1.big_initial() && line1.big_initial()->guesses() ) flag = true;
-    else
-      for( int j = 0 ; j < line1.characters(); ++j )
-        { if( line1.character( j ).guesses() ) { flag = true; break; } }
+    for( int j = 0 ; j < line1.characters(); ++j )
+      { if( line1.character( j ).guesses() ) { flag = true; break; } }
     if( !flag ) delete_line( tlpv, i );
     }
 
