@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 class Page_image;
@@ -21,30 +20,29 @@ class Rational;
 
 class Textline : public Track
   {
-  Character * _big_initial;
+  int _big_initials;
   mutable std::vector< Character * > cpv;
 
 public:
-  Textline() throw() : _big_initial( 0 ) {}
+  Textline() throw() : _big_initials( 0 ) {}
   Textline( const Textline & tl ) throw();
   Textline & operator=( const Textline & tl ) throw();
   ~Textline() throw();
   void set_track() throw();
+  void verify_big_initials() throw();
 
-  const Character * big_initial() const throw() { return _big_initial; }
-  void big_initial( Character * p ) throw();
-//  void delete_big_initial() throw();
-
-  void delete_character( int i ) throw();
-  int  shift_characterp( Character * p ) throw();
-  bool insert_space( int i, bool tab = false ) throw();
-  void join( Textline & tl ) throw();
-
-  Character & character( int i ) const throw();
+  int big_initials() const throw() { return _big_initials; }
+  Character & character( const int i ) const throw();
   int characters() const throw() { return cpv.size(); }
   Rectangle charbox( const Character & c ) const throw();
   int width() const throw()
-    { return cpv.size() ? cpv.back()->right() - cpv[0]->left() : 0; }
+    { return cpv.size() ? cpv.back()->right() - cpv.front()->left() : 0; }
+
+  void delete_character( const int i ) throw();
+  int  shift_characterp( Character * p, const bool big = false ) throw();
+  bool insert_space( const int i, const bool tab = false ) throw();
+  void insert_spaces() throw();
+  void join( Textline & tl ) throw();
 
   int mean_height() const throw();
   Rational mean_width() const throw();
