@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <cctype>
@@ -25,7 +24,7 @@
 #include "track.h"
 #include "ucs.h"
 #include "bitmap.h"
-#include "block.h"
+#include "blob.h"
 #include "character.h"
 #include "textline.h"
 
@@ -45,26 +44,26 @@ int find_space_or_hyphen( const std::vector< Character * > & cpv, int i ) throw(
 
 void Textline::recognize2( const Charset & charset ) throw()
   {
-  if( characters() == 0 ) return;
+  if( big_initials() >= characters() ) return;
 
-  // try to recognize separately the 3 overlapped blocks of an
+  // try to recognize separately the 3 overlapped blobs of an
   // unrecognized character
-  for( int i = 0; i < characters(); ++i )
+  for( int i = big_initials(); i < characters(); ++i )
     {
     Character & c = character( i );
-    if( !c.guesses() && c.blocks() == 3 )
+    if( !c.guesses() && c.blobs() == 3 )
       {
-      const Block & b1 = c.block( 0 );
-      const Block & b2 = c.block( 1 );
-      const Block & b3 = c.block( 2 );		// lower block
+      const Blob & b1 = c.blob( 0 );
+      const Blob & b2 = c.blob( 1 );
+      const Blob & b3 = c.blob( 2 );		// lower blob
       if( Ocrad::similar( b2.height(), b3.height(), 20 ) && !b2.h_overlaps( b3 ) &&
           b2.v_includes( b3.vcenter() ) && b3.v_includes( b2.vcenter() ) &&
           b1.bottom() < b2.top() && b1.bottom() < b3.top() )
         {
-        Character c2( new Block( b2 ) );
-        Character c3( new Block( b3 ) );
-        if( b2.h_includes( b1.hcenter() ) ) c2.shift_blockp( new Block( b1 ) );
-        else if( b3.h_includes( b1.hcenter() ) ) c3.shift_blockp( new Block( b1 ) );
+        Character c2( new Blob( b2 ) );
+        Character c3( new Blob( b3 ) );
+        if( b2.h_includes( b1.hcenter() ) ) c2.shift_blobp( new Blob( b1 ) );
+        else if( b3.h_includes( b1.hcenter() ) ) c3.shift_blobp( new Blob( b1 ) );
         c2.recognize1( charset, charbox( c2 ) );
         c3.recognize1( charset, charbox( c3 ) );
         if( c2.guesses() && c3.guesses() )
@@ -73,17 +72,17 @@ void Textline::recognize2( const Charset & charset ) throw()
       }
     }
 
-  // try to recognize separately the 2 overlapped blocks of an
+  // try to recognize separately the 2 overlapped blobs of an
   // unrecognized character
-  for( int i = 0; i < characters(); ++i )
+  for( int i = big_initials(); i < characters(); ++i )
     {
     Character & c = character( i );
-    if( !c.guesses() && c.blocks() == 2 &&
-        c.block( 0 ).v_overlaps( c.block( 1 ) ) )
+    if( !c.guesses() && c.blobs() == 2 &&
+        c.blob( 0 ).v_overlaps( c.blob( 1 ) ) )
       {
-      Character c1( new Block( c.block( 0 ) ) );
+      Character c1( new Blob( c.blob( 0 ) ) );
       c1.recognize1( charset, charbox( c1 ) );
-      Character c2( new Block( c.block( 1 ) ) );
+      Character c2( new Blob( c.blob( 1 ) ) );
       c2.recognize1( charset, charbox( c2 ) );
       if( ( c1.guesses() && c2.guesses() ) ||
           Ocrad::similar( c1.height(), c2.height(), 20 ) )
@@ -98,28 +97,28 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // remove speckles under the charbox' bottom of an unrecognized character
-  for( int i = 0; i < characters(); ++i )
+  for( int i = big_initials(); i < characters(); ++i )
     {
     Character & c = character( i );
-    if( !c.guesses() && c.blocks() == 2 &&
-        c.block( 0 ).size() > 10 * c.block( 1 ).size() &&
-        c.block( 1 ).top() > charbox( c ).bottom() )
+    if( !c.guesses() && c.blobs() == 2 &&
+        c.blob( 0 ).size() > 10 * c.blob( 1 ).size() &&
+        c.blob( 1 ).top() > charbox( c ).bottom() )
       {
-      Character c1( new Block( c.block( 0 ) ) );
+      Character c1( new Blob( c.blob( 0 ) ) );
       c1.recognize1( charset, charbox( c1 ) );
       if( c1.guesses() ) c = c1;
       }
     }
 
   // remove speckles above the charbox' top of an unrecognized character
-  for( int i = 0; i < characters(); ++i )
+  for( int i = big_initials(); i < characters(); ++i )
     {
     Character & c = character( i );
-    if( !c.guesses() && c.blocks() == 2 &&
-        c.block( 1 ).size() > 5 * c.block( 0 ).size() &&
-        c.block( 0 ).bottom() + 2 * c.block( 0 ).height() < charbox( c ).top() )
+    if( !c.guesses() && c.blobs() == 2 &&
+        c.blob( 1 ).size() > 5 * c.blob( 0 ).size() &&
+        c.blob( 0 ).bottom() + 2 * c.blob( 0 ).height() < charbox( c ).top() )
       {
-      Character c1( new Block( c.block( 1 ) ) );
+      Character c1( new Blob( c.blob( 1 ) ) );
       c1.recognize1( charset, charbox( c1 ) );
       if( c1.guesses() ) c = c1;
       }
@@ -127,20 +126,20 @@ void Textline::recognize2( const Charset & charset ) throw()
 
   // try to separate lightly merged characters
   // FIXME try all possible separation points
-  // FIXME sometimes leaves unconnected blocks
-  for( int i = 0; i < characters(); ++i )
+  // FIXME sometimes leaves unconnected blobs
+  for( int i = big_initials(); i < characters(); ++i )
     {
     Character & c = character( i );
     if( !c.guesses() && c.width() > 20 && 5 * c.width() >= 3 * c.height() &&
         5 * c.height() >= 3 * mean_height() )
       {
-      int ib = c.blocks() - 1;
+      int ib = c.blobs() - 1;
       for( int k = ib - 1; k >= 0; --k )
-        if( c.block( k ).width() > c.block( ib ).width() ) ib = k;
-      if( ib < 0 || 10 * c.block( ib ).width() < 9 * c.width() ||
-          c.block( ib ).bottom() < c.bottom() )
+        if( c.blob( k ).width() > c.blob( ib ).width() ) ib = k;
+      if( ib < 0 || 10 * c.blob( ib ).width() < 9 * c.width() ||
+          c.blob( ib ).bottom() < c.bottom() )
         continue;
-      const Block & b = c.block( ib );			// widest block
+      const Blob & b = c.blob( ib );			// widest blob
       int colmin = 0, cmin = b.height() + 1;
       for( int col = b.hpos( 30 ); col <= b.hpos( 70 ); ++col )
         {
@@ -156,18 +155,18 @@ void Textline::recognize2( const Charset & charset ) throw()
       if( colmin <= b.left() || colmin >= b.right() ) continue;
       Rectangle r1( b.left(), b.top(), colmin - 1, b.bottom() );
       Rectangle r2( colmin + 1, b.top(), b.right(), b.bottom() );
-      Block b1( b, r1 );
+      Blob b1( b, r1 );
       b1.adjust_height(); if( 2 * b1.height() < b.height() ) continue;
-      Block b2( b, r2 );
+      Blob b2( b, r2 );
       b2.adjust_height(); if( 2 * b2.height() < b.height() ) continue;
       b1.find_holes(); b2.find_holes();
-      Character c1( new Block( b1 ) );
-      Character c2( new Block( b2 ) );
-      for( int j = 0; j < c.blocks(); ++j ) if( j != ib )
+      Character c1( new Blob( b1 ) );
+      Character c2( new Blob( b2 ) );
+      for( int j = 0; j < c.blobs(); ++j ) if( j != ib )
         {
-        const Block & bj = c.block( j );
-        if( c1.includes_hcenter( bj ) ) c1.shift_blockp( new Block( bj ) );
-        else if( c2.includes_hcenter( bj ) ) c2.shift_blockp( new Block( bj ) );
+        const Blob & bj = c.blob( j );
+        if( c1.includes_hcenter( bj ) ) c1.shift_blobp( new Blob( bj ) );
+        else if( c2.includes_hcenter( bj ) ) c2.shift_blobp( new Blob( bj ) );
         }
       c1.recognize1( charset, charbox( c1 ) );
       c2.recognize1( charset, charbox( c2 ) );
@@ -180,19 +179,19 @@ void Textline::recognize2( const Charset & charset ) throw()
       }
     }
 
-  // try to recognize 1 block unrecognized characters with holes by
+  // try to recognize 1 blob unrecognized characters with holes by
   // removing small holes (noise)
-  for( int i = 0; i < characters(); ++i )
+  for( int i = big_initials(); i < characters(); ++i )
     {
     Character & c = character( i );
-    if( !c.guesses() && c.blocks() == 1 && c.block( 0 ).holes() )
+    if( !c.guesses() && c.blobs() == 1 && c.blob( 0 ).holes() )
       {
       Character c1( c );
-      Block & b = c1.block( 0 );
+      Blob & b = c1.blob( 0 );
       for( int j = b.holes() - 1; j >= 0; --j )
         if( 64 * b.hole( j ).size() <= b.size() ||
             16 * b.hole( j ).height() <= b.height() ) b.fill_hole( j );
-      if( b.holes() < c.block( 0 ).holes() )
+      if( b.holes() < c.blob( 0 ).holes() )
         {
         c1.recognize1( charset, charbox( c1 ) );
         if( c1.guesses() ) { c = c1; continue; }
@@ -208,24 +207,24 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // separate merged characters recognized by recognize1
-  for( int i = 0; i < characters(); )		// FIXME this block sucks
+  for( int i = big_initials(); i < characters(); )		// FIXME this block sucks
     {
     if( !cpv[i]->guesses() || cpv[i]->guess( 0 ).code != 0 ) { ++i; continue; }
     Character c( *cpv[i] );
     delete_character( i );
-    if( c.guesses() >= 3 && c.blocks() >= 1 )
+    if( c.guesses() >= 3 && c.blobs() >= 1 )
       {
       int left = c.guess( 0 ).value;
       for( int g = 1; g < c.guesses(); ++g )
         {
-        Block & b = c.block( c.blocks() - 1 );
+        Blob & b = c.blob( c.blobs() - 1 );
         Rectangle re( left, b.top(), c.guess( g ).value, b.bottom() );
         b.add_rectangle( re );
-        Block b1( b, re ); b1.adjust_height(); b1.find_holes();
-        Character c1( new Block( b1 ) );
-        for( int k = 0; k < c.blocks() - 1; ++k )
-          if( !c.block( k ).includes( re ) && re.includes_hcenter( c.block( k ) ) )
-            c1.shift_blockp( new Block( c.block( k ) ) );
+        Blob b1( b, re ); b1.adjust_height(); b1.find_holes();
+        Character c1( new Blob( b1 ) );
+        for( int k = 0; k < c.blobs() - 1; ++k )
+          if( !c.blob( k ).includes( re ) && re.includes_hcenter( c.blob( k ) ) )
+            c1.shift_blobp( new Blob( c.blob( k ) ) );
         c1.add_guess( c.guess( g ).code, 0 );
         shift_characterp( new Character( c1 ) );
         left = re.right() + 1;
@@ -234,7 +233,7 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // choose between 'B' and 'a'
-  for( int i = 0, begin = 0; i < characters(); ++i )
+  for( int i = big_initials(), begin = i; i < characters(); ++i )
     {
     Character & c1 = character( i );
     if( c1.maybe(' ') ) { begin = i + 1 ; continue; }
@@ -262,7 +261,7 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // choose between '8' and 'a' or 'e'
-  for( int i = 0, begin = 0; i < characters(); ++i )
+  for( int i = big_initials(), begin = i; i < characters(); ++i )
     {
     Character & c1 = character( i );
     if( c1.maybe(' ') ) { begin = i + 1 ; continue; }
@@ -291,15 +290,15 @@ void Textline::recognize2( const Charset & charset ) throw()
 
   // transform some small letters to capitals
   {
-  int begin = 0;
+  int begin = big_initials();
   bool isolated = false;	// isolated letters compare with all line
-  for( int i = 0; i < characters(); ++i )
+  for( int i = big_initials(); i < characters(); ++i )
     {
     Character & c1 = character( i );
     if( c1.maybe(' ') )
       {
       if( i + 2 < characters() && character( i + 2 ).maybe(' ') )
-        { begin = 0; isolated = true; }
+        { begin = big_initials(); isolated = true; }
       else { begin = i + 1; isolated = false; }
       continue;
       }
@@ -339,14 +338,14 @@ void Textline::recognize2( const Charset & charset ) throw()
   }
 
   // transform 'i' into 'j'
-  for( int i = 0; i < characters(); ++i )
+  for( int i = big_initials(); i < characters(); ++i )
     {
     Character & c1 = character( i );
     if( c1.guesses() == 1 && c1.guess( 0 ).code == 'i' )
       {
       int j = i + 1;
       if( j >= characters() || !character( j ).guesses() )
-        { j = i - 1; if( j < 0 || !character( j ).guesses() ) continue; }
+        { j = i - 1; if( j < big_initials() || !character( j ).guesses() ) continue; }
       Character & c2 = character( j );
       if( UCS::isvowel( c2.guess( 0 ).code ) &&
           c1.bottom() >= c2.bottom() + ( c2.height() / 4 ) )
@@ -356,9 +355,9 @@ void Textline::recognize2( const Charset & charset ) throw()
 
   // transform small o or u with accent or diaeresis to capital
   {
-  int begin = 0;
+  int begin = big_initials();
   bool isolated = false;	// isolated letters compare with all line
-  for( int i = 0; i < characters(); ++i )
+  for( int i = big_initials(); i < characters(); ++i )
     {
     Character & c1 = character( i );
     if( c1.guesses() >= 1 )
@@ -366,15 +365,15 @@ void Textline::recognize2( const Charset & charset ) throw()
       if( c1.maybe(' ') )
         {
         if( i + 2 < characters() && character( i + 2 ).maybe(' ') )
-          { begin = 0; isolated = true; }
+          { begin = big_initials(); isolated = true; }
         else { begin = i + 1; isolated = false; }
         continue;
         }
       int code = c1.guess( 0 ).code;
-      if( code < 128 || c1.blocks() < 2 ) continue;
+      if( code < 128 || c1.blobs() < 2 ) continue;
       int codeb = UCS::base_letter( code );
       if( codeb != 'o' && codeb != 'u' ) continue;
-      const Block & b1 = c1.block( c1.blocks() - 1 );	// lower block
+      const Blob & b1 = c1.blob( c1.blobs() - 1 );	// lower blob
       for( int j = begin; j < characters(); ++j ) if( j != i )
         {
         Character & c2 = character( j );
@@ -396,7 +395,7 @@ void Textline::recognize2( const Charset & charset ) throw()
   }
 
   // transform 'O' or 'l' into '0' or '1'
-  for( int i = 0, begin = 0; i < characters(); ++i )
+  for( int i = big_initials(), begin = i; i < characters(); ++i )
     {
     Character & c1 = character( i );
     if( c1.maybe(' ') ) { begin = i + 1 ; continue; }
@@ -425,7 +424,7 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // transform a small 'p' to a capital 'P'
-  for( int i = characters() - 1; i >= 0; --i )
+  for( int i = characters() - 1; i >= big_initials(); --i )
     {
     Character & c1 = character( i );
     if( c1.guesses() == 1 && c1.guess( 0 ).code == 'p' )
@@ -447,14 +446,14 @@ void Textline::recognize2( const Charset & charset ) throw()
             }
           }
         }
-      if( !valid_c2 && i > 0 && !character(i-1).maybe(' ') )
+      if( !valid_c2 && i > big_initials() && !character(i-1).maybe(' ') )
         cap = ( std::abs( c1.bottom() - charbox(c1).bottom() ) <= 2 );
       if( cap ) c1.only_guess( 'P', 0 );
       }
     }
 
   // transform a capital 'Y' to a small 'y'
-  for( int i = characters() - 1; i > 0; --i )
+  for( int i = characters() - 1; i > big_initials(); --i )
     {
     Character & c1 = character( i - 1 );
     if( c1.guesses() == 1 && c1.guess( 0 ).code == 'Y' )
@@ -478,12 +477,12 @@ void Textline::recognize2( const Charset & charset ) throw()
 
   // transform a SSCEDI to a CSCEDI
   if( charset.enabled( Charset::iso_8859_9 ) )
-    for( int i = 0; i < characters(); ++i )
+    for( int i = big_initials(); i < characters(); ++i )
       {
       Character & c = character( i );
       if( c.guesses() == 1 && c.guess( 0 ).code == UCS::SSCEDI )
         {
-        if( i > 0 && character( i - 1 ).guesses() )
+        if( i > big_initials() && character( i - 1 ).guesses() )
           {
           Character & c1 = character( i - 1 );
           int code = c1.guess( 0 ).code;
@@ -505,7 +504,7 @@ void Textline::recognize2( const Charset & charset ) throw()
       }
 
   // transform words like 'lO.OOO' into numbers like '10.000'
-  for( int begin = 0, end = 0; begin < characters(); begin = end + 1 )
+  for( int begin = big_initials(), end = begin; begin < characters(); begin = end + 1 )
     {
     end = find_space_or_hyphen( cpv, begin );
     if( end - begin < 2 ) continue;
@@ -544,7 +543,7 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // detects Roman numerals 'II', 'III' and 'IIII'
-  for( int begin = 0, end = 0; begin < characters(); begin = end + 1 )
+  for( int begin = big_initials(), end = begin; begin < characters(); begin = end + 1 )
     {
     end = find_space_or_hyphen( cpv, begin );
     if( end - begin < 2 || end - begin > 4 ) continue;
@@ -565,7 +564,7 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // transform a vertical bar into 'l' or 'I' (or a 'l' into an 'I')
-  for( int i = 0; i < characters(); ++i )
+  for( int i = big_initials(); i < characters(); ++i )
     {
     Character & c = character( i );
     if( c.guesses() != 1 ) continue;
@@ -573,11 +572,8 @@ void Textline::recognize2( const Charset & charset ) throw()
     if( code == '|' || code == 'l' )
       {
       int lcode = 0, rcode = 0;
-      if( i > 0 )
-        { if( character( i - 1 ).guesses() )
-            lcode = character( i - 1 ).guess( 0 ).code; }
-      else if( _big_initial && _big_initial->guesses() )
-        lcode = _big_initial->guess( 0 ).code;
+      if( i > 0 && character( i - 1 ).guesses() )
+        lcode = character( i - 1 ).guess( 0 ).code;
       if( i < characters() - 1 && character( i + 1 ).guesses() )
         rcode = character( i + 1 ).guess( 0 ).code;
       if( UCS::isupper( rcode ) &&
@@ -600,7 +596,7 @@ void Textline::recognize2( const Charset & charset ) throw()
 
   // transform 'l' or '|' into UCS::SINODOT
   if( charset.enabled( Charset::iso_8859_9 ) )
-  for( int i = 0, begin = 0; i < characters(); ++i )
+  for( int i = big_initials(), begin = i; i < characters(); ++i )
     {
     Character & c1 = character( i );
     if( c1.maybe(' ') ) { begin = i + 1 ; continue; }
@@ -641,7 +637,7 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // join two adjacent single quotes into a double quote
-  for( int i = 0; i < characters() - 1; ++i )
+  for( int i = big_initials(); i < characters() - 1; ++i )
     {
     Character & c1 = character( i );
     Character & c2 = character( i + 1 );
@@ -656,7 +652,7 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // join a comma followed by a period into a semicolon
-  for( int i = 0; i < characters() - 1; ++i )
+  for( int i = big_initials(); i < characters() - 1; ++i )
     {
     Character & c1 = character( i );
     Character & c2 = character( i + 1 );
@@ -671,7 +667,7 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // choose between 'a' and 'Q'
-  for( int i = 0, begin = 0; i < characters(); ++i )
+  for( int i = big_initials(), begin = i; i < characters(); ++i )
     {
     Character & c = character( i );
     if( c.maybe(' ') ) { begin = i + 1 ; continue; }
@@ -706,7 +702,7 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // join a 'n' followed by a 'I' into a 'm'
-  for( int i = 0; i < characters() - 1; ++i )
+  for( int i = big_initials(); i < characters() - 1; ++i )
     {
     Character & c1 = character( i );
     Character & c2 = character( i + 1 );
@@ -722,7 +718,7 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // join the secuence '', '/', 'o', ' ' into a '%'
-  for( int i = 0; i < characters() - 3; ++i )
+  for( int i = big_initials(); i < characters() - 3; ++i )
     {
     Character & c1 = character( i );
     if( c1.guesses() == 1 && c1.guess( 0 ).code == UCS::DEG )
