@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <cstdio>
@@ -24,7 +23,7 @@
 #include "rectangle.h"
 #include "track.h"
 #include "bitmap.h"
-#include "block.h"
+#include "blob.h"
 #include "character.h"
 #include "page_image.h"
 #include "textline.h"
@@ -37,26 +36,26 @@ namespace {
 struct Zone
   {
   Rectangle rectangle;
-  std::vector< std::vector< Block * > > blockp_matrix;
+  std::vector< std::vector< Blob * > > blobp_matrix;
   Zone( const Rectangle & r ) : rectangle( r ) {}
   };
 
 
-int blocks_in_zone( const std::vector< Zone > & zone_vector, int i ) throw()
+int blobs_in_zone( const std::vector< Zone > & zone_vector, int i ) throw()
   {
-  const std::vector< std::vector< Block * > > & blockp_matrix = zone_vector[i].blockp_matrix;
+  const std::vector< std::vector< Blob * > > & blobp_matrix = zone_vector[i].blobp_matrix;
   int sum = 0;
-  for( unsigned int cut = 0; cut < blockp_matrix.size(); ++cut )
-    sum += blockp_matrix[cut].size();
+  for( unsigned int cut = 0; cut < blobp_matrix.size(); ++cut )
+    sum += blobp_matrix[cut].size();
   return sum;
   }
 
 
-int blocks_in_page( const std::vector< Zone > & zone_vector ) throw()
+int blobs_in_page( const std::vector< Zone > & zone_vector ) throw()
   {
   int sum = 0;
   for( unsigned int i = 0; i < zone_vector.size(); ++i )
-    sum += blocks_in_zone( zone_vector, i );
+    sum += blobs_in_zone( zone_vector, i );
   return sum;
   }
 
@@ -65,126 +64,138 @@ void bprint( const std::vector< Zone > & zone_vector, FILE * outfile ) throw()
   {
 //  std::fprintf( outfile, "page size %dw x %dh\n", width(), height() );
   std::fprintf( outfile, "total zones in page %d\n", (int)zone_vector.size() );
-  std::fprintf( outfile, "total blocks in page %d\n\n", blocks_in_page( zone_vector ) );
+  std::fprintf( outfile, "total blobs in page %d\n\n", blobs_in_page( zone_vector ) );
   for( unsigned int zindex = 0; zindex < zone_vector.size(); ++zindex )
     {
     const Rectangle & r = zone_vector[zindex].rectangle;
-    const std::vector< std::vector< Block * > > & blockp_matrix = zone_vector[zindex].blockp_matrix;
+    const std::vector< std::vector< Blob * > > & blobp_matrix = zone_vector[zindex].blobp_matrix;
 
     std::fprintf( outfile, "zone %d of %d\n", zindex + 1, (int)zone_vector.size() );
     std::fprintf( outfile, "zone size %dw x %dh\n", r.width(), r.height() );
-    std::fprintf( outfile, "total cuts in zone %d\n", (int)blockp_matrix.size() );
-    std::fprintf( outfile, "total blocks in zone %d\n\n", blocks_in_zone( zone_vector, zindex ) );
+    std::fprintf( outfile, "total cuts in zone %d\n", (int)blobp_matrix.size() );
+    std::fprintf( outfile, "total blobs in zone %d\n\n", blobs_in_zone( zone_vector, zindex ) );
 
-    for( unsigned int cut = 0; cut < blockp_matrix.size(); ++cut )
+    for( unsigned int cut = 0; cut < blobp_matrix.size(); ++cut )
       {
-      std::fprintf( outfile, "cut %d blocks %d\n", cut + 1, (int)blockp_matrix[cut].size() );
-      for( unsigned int i = 0; i < blockp_matrix[cut].size(); ++i )
-      blockp_matrix[cut][i]->print( outfile );
+      std::fprintf( outfile, "cut %d blobs %d\n", cut + 1, (int)blobp_matrix[cut].size() );
+      for( unsigned int i = 0; i < blobp_matrix[cut].size(); ++i )
+      blobp_matrix[cut][i]->print( outfile );
       }
     }
   }
 
 
-inline void join_blocks( std::vector< Block * > & blockp_vector,
-                         std::vector< Block * > & v1,
-                         std::vector< Block * > & v2,
-                         Block * p1, Block * p2, int i ) throw()
+inline void join_blobs( std::vector< Blob * > & blobp_vector,
+                         std::vector< Blob * > & v1,
+                         std::vector< Blob * > & v2,
+                         Blob * p1, Blob * p2, int i ) throw()
   {
   if( p1->top() > p2->top() )
     {
-    Block * temp = p1; p1 = p2; p2 = temp;
+    Blob * temp = p1; p1 = p2; p2 = temp;
     std::replace( v2.begin(), v2.begin() + i + 1, p2, p1 );
     }
   else std::replace( v1.begin() + i, v1.end(), p2, p1 );
 
-  i = blockp_vector.size();
-  while( --i >= 0 && blockp_vector[i] != p2 );
-  if( i < 0 ) Ocrad::internal_error( "join_blocks, lost block" );
-  blockp_vector.erase( blockp_vector.begin() + i );
+  i = blobp_vector.size();
+  while( --i >= 0 && blobp_vector[i] != p2 );
+  if( i < 0 ) Ocrad::internal_error( "join_blobs, lost blob" );
+  blobp_vector.erase( blobp_vector.begin() + i );
 
   p1->add_bitmap( *p2 );
   delete p2;
   }
 
 
-void ignore_abnormal_blocks( std::vector< Block * > & blockp_vector ) throw()
+void ignore_abnormal_blobs( std::vector< Blob * > & blobp_vector ) throw()
   {
-  for( int i = blockp_vector.size() - 1; i >= 0; --i )
+  for( int i = blobp_vector.size() - 1; i >= 0; --i )
     {
-    Block & b = *blockp_vector[i];
+    Blob & b = *blobp_vector[i];
     if( b.height() > 35 * b.width() || b.width() > 25 * b.height() )
-      { delete blockp_vector[i];
-      blockp_vector.erase( blockp_vector.begin() + i ); }
+      { delete blobp_vector[i];
+      blobp_vector.erase( blobp_vector.begin() + i ); }
     }
   }
 
 
-void ignore_small_blocks( std::vector< Block * > & blockp_vector ) throw()
+void ignore_small_blobs( std::vector< Blob * > & blobp_vector ) throw()
   {
-  int to = 0, blocks = blockp_vector.size();
-  for( int from = 0; from < blocks; ++from )
+  int to = 0, blobs = blobp_vector.size();
+  for( int from = 0; from < blobs; ++from )
     {
-    Block * p = blockp_vector[from];
+    Blob * p = blobp_vector[from];
     if( p->height() > 4 || p->width() > 4 ||
         ( ( p->height() > 2 || p->width() > 2 ) && p->area() > 5 ) )
-      { blockp_vector[from] = blockp_vector[to]; blockp_vector[to] = p; ++to; }
+      { blobp_vector[from] = blobp_vector[to]; blobp_vector[to] = p; ++to; }
     }
-  if( to < blocks )
+  if( to < blobs )
     {
-    for( int i = to; i < blocks; ++i ) delete blockp_vector[i];
-    blockp_vector.erase( blockp_vector.begin() + to, blockp_vector.end() );
+    for( int i = to; i < blobs; ++i ) delete blobp_vector[i];
+    blobp_vector.erase( blobp_vector.begin() + to, blobp_vector.end() );
     }
   }
 
 
-void ignore_wide_blocks( Zone & zone ) throw()
+void ignore_wide_blobs( Zone & zone ) throw()
   {
   const Rectangle & r = zone.rectangle;
-  std::vector< std::vector< Block * > > & blockp_matrix = zone.blockp_matrix;
+  std::vector< std::vector< Blob * > > & blobp_matrix = zone.blobp_matrix;
 
-  for( unsigned int cut = 0; cut < blockp_matrix.size(); ++cut )
+  for( unsigned int cut = 0; cut < blobp_matrix.size(); ++cut )
     {
     bool frame_found = false;
-    for( unsigned int i = 0; i < blockp_matrix[cut].size(); )
+    for( unsigned int i = 0; i < blobp_matrix[cut].size(); )
       {
-      std::vector< Block * > & blockp_vector = blockp_matrix[cut];
-      const Block & b = *blockp_vector[i];
+      std::vector< Blob * > & blobp_vector = blobp_matrix[cut];
+      Blob & b = *blobp_vector[i];
       if( 2 * b.width() < r.width() ) { ++i; continue; }
-      blockp_vector.erase( blockp_vector.begin() + i );
-      if( 4 * b.area() > b.size() )		// image, not frame
+      blobp_vector.erase( blobp_vector.begin() + i );
+      if( 4 * b.area() <= 3 * b.size() )
         {
-        if( 5 * b.width() > 4 * r.width() && 5 * b.height() > 4 * r.height() )
-          {
-          for( unsigned int j = 0; j < blockp_vector.size(); ++j )
-            delete blockp_vector[j];
-          blockp_vector.clear(); break;
-          }
-        for( int j = blockp_vector.size() - 1; j >= 0; --j )
-          {
-          const Block & b2 = *blockp_vector[j];
-          if( b.includes( b2 ) )
-            { delete &b2; blockp_vector.erase( blockp_vector.begin() + j ); }
-          else if( b2.top() < b.top() ) break;
-          }
+        if( 4 * b.area() <= b.size() )
+          { frame_found = true; delete &b; continue; }
+        b.find_holes();
+        int j = -1;
+        if( b.holes() < std::min( b.height(), b.width() ) )
+          for( j = 0; j < b.holes(); ++j )
+            if( 4 * b.hole( j ).size() >= b.size() &&
+                4 * b.hole( j ).area() >= b.size() )
+              break;
+        if( j >= 0 && j < b.holes() )
+          { frame_found = true; delete &b; continue; }
         }
-      else frame_found = true;
+      // image, not frame
+      if( 5 * b.width() > 4 * r.width() && 5 * b.height() > 4 * r.height() )
+        {
+        for( unsigned int j = 0; j < blobp_vector.size(); ++j )
+          delete blobp_vector[j];
+        blobp_vector.clear(); delete &b; break;
+        }
+      for( int j = blobp_vector.size() - 1; j >= 0; --j )
+        {
+        const Blob & b2 = *blobp_vector[j];
+        if( b.includes( b2 ) )
+          { delete &b2; blobp_vector.erase( blobp_vector.begin() + j ); }
+        else if( b2.top() < b.top() ) break;
+        }
+      delete &b;
       }
-    if( frame_found )	// Make cuts from blocks inside deleted frame(s)
+    if( frame_found )	// Make cuts from blobs inside deleted frame(s)
       {
       int bottom = 0;
-      for( unsigned int i = 0; i < blockp_matrix[cut].size(); ++i )
+      for( unsigned int i = 0; i < blobp_matrix[cut].size(); ++i )
         {
-        std::vector< Block * > & blockp_vector = blockp_matrix[cut];
-        const Block & b = *blockp_vector[i];
+        std::vector< Blob * > & blobp_vector = blobp_matrix[cut];
+        const Blob & b = *blobp_vector[i];
         if( b.bottom() > bottom )
           {
           int old_bottom = bottom; bottom = b.bottom();
           if( b.top() > old_bottom && i > 0 )
             {
-            std::vector< Block * > new_blockp_vector( blockp_vector.begin() + i, blockp_vector.end() );
-            blockp_vector.erase( blockp_vector.begin() + i, blockp_vector.end() );
-            blockp_matrix.insert( blockp_matrix.begin() + cut + 1, new_blockp_vector );
+            std::vector< Blob * > new_blobp_vector( blobp_vector.begin() + i, blobp_vector.end() );
+            blobp_vector.erase( blobp_vector.begin() + i, blobp_vector.end() );
+            blobp_matrix.insert( blobp_matrix.begin() + cut + 1, new_blobp_vector );
             ++cut; i = 0;
             }
           }
@@ -194,12 +205,12 @@ void ignore_wide_blocks( Zone & zone ) throw()
   }
 
 
-void remove_top_bottom_noise( std::vector< Block * > & blockp_vector ) throw()
+void remove_top_bottom_noise( std::vector< Blob * > & blobp_vector ) throw()
   {
-  int blocks = blockp_vector.size();
-  for( int i = 0; i < blocks; ++i )
+  int blobs = blobp_vector.size();
+  for( int i = 0; i < blobs; ++i )
     {
-    Block & b = *blockp_vector[i];
+    Blob & b = *blobp_vector[i];
     if( b.height() < 11 ) continue;
 
     int c = 0;
@@ -215,12 +226,12 @@ void remove_top_bottom_noise( std::vector< Block * > & blockp_vector ) throw()
   }
 
 
-void remove_left_right_noise( std::vector< Block * > & blockp_vector ) throw()
+void remove_left_right_noise( std::vector< Blob * > & blobp_vector ) throw()
   {
-  int blocks = blockp_vector.size();
-  for( int i = 0; i < blocks; ++i )
+  int blobs = blobp_vector.size();
+  for( int i = 0; i < blobs; ++i )
     {
-    Block & b = *blockp_vector[i];
+    Blob & b = *blobp_vector[i];
     if( b.width() < 6 ) continue;
 
     int c = 0;
@@ -240,12 +251,12 @@ void find_holes( std::vector< Zone > & zone_vector ) throw()
   {
   for( unsigned int zi = 0; zi < zone_vector.size(); ++zi )
     {
-    std::vector< std::vector< Block * > > & blockp_matrix = zone_vector[zi].blockp_matrix;
-    for( unsigned int bmi = 0; bmi < blockp_matrix.size(); ++bmi )
+    std::vector< std::vector< Blob * > > & blobp_matrix = zone_vector[zi].blobp_matrix;
+    for( unsigned int bmi = 0; bmi < blobp_matrix.size(); ++bmi )
       {
-      std::vector< Block * > & blockp_vector = blockp_matrix[bmi];
-      for( unsigned int bvi = 0; bvi < blockp_vector.size(); ++bvi )
-        blockp_vector[bvi]->find_holes();
+      std::vector< Blob * > & blobp_vector = blobp_matrix[bmi];
+      for( unsigned int bvi = 0; bvi < blobp_vector.size(); ++bvi )
+        blobp_vector[bvi]->find_holes();
       }
     }
   }
@@ -259,10 +270,10 @@ void scan_page( const Page_image & page_image,
     zone_vector.push_back( Zone( page_image.rectangle( zindex ) ) );
     const Rectangle & re = zone_vector[zindex].rectangle;
     const int zthreshold = page_image.threshold( zindex );
-    std::vector< std::vector< Block * > > & blockp_matrix = zone_vector[zindex].blockp_matrix;
-    std::vector< Block * > blockp_vector;
-    std::vector< Block * > old_data( re.width(), (Block *) 0 );
-    std::vector< Block * > new_data( re.width(), (Block *) 0 );
+    std::vector< std::vector< Blob * > > & blobp_matrix = zone_vector[zindex].blobp_matrix;
+    std::vector< Blob * > blobp_vector;
+    std::vector< Blob * > old_data( re.width(), (Blob *) 0 );
+    std::vector< Blob * > new_data( re.width(), (Blob *) 0 );
 
     for( int row = re.top() + 1; row <= re.bottom(); ++row )
       {
@@ -276,61 +287,61 @@ void scan_page( const Page_image & page_image,
         else					// black point
           {
           blank_row = false;
-          Block *p;
-          Block *lp  = new_data[dcol-1];
-          Block *ltp = old_data[dcol-1];
-          Block *tp  = old_data[dcol];
-          Block *rtp = old_data[dcol+1];
+          Blob *p;
+          Blob *lp  = new_data[dcol-1];
+          Blob *ltp = old_data[dcol-1];
+          Blob *tp  = old_data[dcol];
+          Blob *rtp = old_data[dcol+1];
           if( lp )       { p = lp;  p->add_point( row, col ); }
           else if( ltp ) { p = ltp; p->add_point( row, col ); }
           else if( tp )  { p = tp;  p->add_point( row, col ); }
           else if( rtp ) { p = rtp; p->add_point( row, col ); }
           else
             {
-            p = new Block( col, row, col, row );
+            p = new Blob( col, row, col, row );
             p->set_bit( row, col, true );
-            blockp_vector.push_back( p );
+            blobp_vector.push_back( p );
             }
           new_data[dcol] = p;
           if( rtp && p != rtp )
-            join_blocks( blockp_vector, old_data, new_data, p, rtp, dcol );
+            join_blobs( blobp_vector, old_data, new_data, p, rtp, dcol );
           }
         }
-      if( blank_row && blockp_vector.size() )
+      if( blank_row && blobp_vector.size() )
         {
-        blockp_matrix.push_back( std::vector< Block * >() );
-        blockp_vector.swap( blockp_matrix.back() );
+        blobp_matrix.push_back( std::vector< Blob * >() );
+        blobp_vector.swap( blobp_matrix.back() );
         }
       }
-    if( blockp_vector.size() )
+    if( blobp_vector.size() )
       {
-      blockp_matrix.push_back( std::vector< Block * >() );
-      blockp_vector.swap( blockp_matrix.back() );
+      blobp_matrix.push_back( std::vector< Blob * >() );
+      blobp_vector.swap( blobp_matrix.back() );
       }
     }
 
   if( debug_level <= 99 )
     for( unsigned int zindex = 0; zindex < zone_vector.size(); ++zindex )
       {
-      ignore_wide_blocks( zone_vector[zindex] );
-      std::vector< std::vector< Block * > > & blockp_matrix = zone_vector[zindex].blockp_matrix;
-      for( unsigned int cut = 0; cut < blockp_matrix.size(); ++cut )
+      ignore_wide_blobs( zone_vector[zindex] );
+      std::vector< std::vector< Blob * > > & blobp_matrix = zone_vector[zindex].blobp_matrix;
+      for( unsigned int cut = 0; cut < blobp_matrix.size(); ++cut )
         {
-        std::vector< Block * > & blockp_vector = blockp_matrix[cut];
-        ignore_small_blocks( blockp_vector );
-        ignore_abnormal_blocks( blockp_vector );
-        remove_top_bottom_noise( blockp_vector );
-        remove_left_right_noise( blockp_vector );
+        std::vector< Blob * > & blobp_vector = blobp_matrix[cut];
+        ignore_small_blobs( blobp_vector );
+        ignore_abnormal_blobs( blobp_vector );
+        remove_top_bottom_noise( blobp_vector );
+        remove_left_right_noise( blobp_vector );
         }
       }
 
   for( int zindex = zone_vector.size() - 1; zindex >= 0; --zindex )
     {
-    std::vector< std::vector< Block * > > & blockp_matrix = zone_vector[zindex].blockp_matrix;
-    for( int cut = blockp_matrix.size() - 1; cut >= 0; --cut )
-      if( !blockp_matrix[cut].size() )
-        blockp_matrix.erase( blockp_matrix.begin() + cut );
-    if( !blockp_matrix.size() )
+    std::vector< std::vector< Blob * > > & blobp_matrix = zone_vector[zindex].blobp_matrix;
+    for( int cut = blobp_matrix.size() - 1; cut >= 0; --cut )
+      if( !blobp_matrix[cut].size() )
+        blobp_matrix.erase( blobp_matrix.begin() + cut );
+    if( !blobp_matrix.size() )
       zone_vector.erase( zone_vector.begin() + zindex );
     }
 
@@ -361,7 +372,7 @@ Textpage::Textpage( const Page_image & page_image,
   for( unsigned int i = 0; i < zone_vector.size(); ++i )
     {
     Textblock * tbp = new Textblock( zone_vector[i].rectangle,
-                                     zone_vector[i].blockp_matrix );
+                                     zone_vector[i].blobp_matrix );
     if( debug_level < 90 )
       tbp->recognize( control.charset, control.filter );
     if( tbp->textlines() ) tbpv.push_back( tbp );
