@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 class Vrhomboid			// Rhomboid with two vertical sides.
@@ -21,7 +20,7 @@ class Vrhomboid			// Rhomboid with two vertical sides.
   int _left, _lvcenter, _right, _rvcenter, _height;
 
 public:
-  Vrhomboid( int l, int lc, int r, int rc, int h ) throw();
+  Vrhomboid( const int l, const int lc, const int r, const int rc, const int h ) throw();
 
   int left()     const throw() { return _left;     }
   int lvcenter() const throw() { return _lvcenter; }
@@ -31,21 +30,20 @@ public:
   int width()    const throw() { return _right - _left + 1; }
   int size()     const throw() { return _height * width(); }
 
-  void left( int l ) throw();
-  void lvcenter( int lc ) throw() { _lvcenter = lc; }
-  void right( int r ) throw();
-  void rvcenter( int rc ) throw() { _rvcenter = rc; }
-  void height( int h ) throw();
-  void extend_left( int l ) throw();
-  void extend_right( int r ) throw();
+  void left( const int l ) throw();
+  void lvcenter( const int lc ) throw() { _lvcenter = lc; }
+  void right( const int r ) throw();
+  void rvcenter( const int rc ) throw() { _rvcenter = rc; }
+  void height( const int h ) throw();
+  void extend_left( const int l ) throw();
+  void extend_right( const int r ) throw();
 
-  int bottom( int col ) const throw() { return vcenter( col ) + ( _height / 2 ); }
-  int top( int col ) const throw() { return bottom( col ) - _height + 1; }
-  int vcenter( int col ) const throw();
+  int bottom( const int col ) const throw() { return vcenter( col ) + ( _height / 2 ); }
+  int top( const int col ) const throw() { return bottom( col ) - _height + 1; }
+  int vcenter( const int col ) const throw();
 
   bool includes( const Rectangle & r ) const throw();
-  bool includes( int row, int col ) const throw();
-  static int vcenter( int bot, int hei ) throw() { return bot - ( hei / 2 ); }
+  bool includes( const int row, const int col ) const throw();
   };
 
 
@@ -66,7 +64,10 @@ public:
   int left() const throw() { return data.size() ? data[0].left() : 0; }
   int right() const throw() { return data.size() ? data.back().right() : 0; }
 
-  int vcenter( int col ) const throw();
-  int bottom( int col ) const throw();
-  int top( int col ) const throw();
+  int bottom( const int col ) const throw();
+  int top( const int col ) const throw();
+  int vcenter( const int col ) const throw();
+
+  bool includes( const Rectangle & r ) const throw();
+  bool includes( const int row, const int col ) const throw();
   };
