@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,15 +12,15 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <cctype>
+
 #include "ucs.h"
 
 
-int UCS::base_letter( int code ) throw()
+int UCS::base_letter( const int code ) throw()
   {
   switch( code )
     {
@@ -88,7 +88,7 @@ int UCS::base_letter( int code ) throw()
   }
 
 
-int UCS::compose( int letter, int accent ) throw()
+int UCS::compose( const int letter, const int accent ) throw()
   {
   switch( letter )
     {
@@ -101,6 +101,7 @@ int UCS::compose( int letter, int accent ) throw()
               if( accent == '^' ) return CECIRCU;
               if( accent == ':' ) return CEDIAER; break;
     case 'G': return CGBREVE;
+    case '[':
     case 'I': if( accent == '\'') return CIACUTE;
               if( accent == '`' ) return CIGRAVE;
               if( accent == '^' ) return CICIRCU;
@@ -128,6 +129,7 @@ int UCS::compose( int letter, int accent ) throw()
     case '9':
     case 'g': return SGBREVE;
     case '|':
+    case ']':
     case 'i':
     case 'l': if( accent == '\'') return SIACUTE;
               if( accent == '`' ) return SIGRAVE;
@@ -152,27 +154,49 @@ int UCS::compose( int letter, int accent ) throw()
   }
 
 
-bool UCS::isalnum( int code ) throw()
+bool UCS::isalnum( const int code ) throw()
   {
-  return ( UCS::isalpha( code ) || ( code < 128 && std::isdigit( code ) ) );
+  return ( UCS::isalpha( code ) || UCS::isdigit( code ) );
   }
 
 
-bool UCS::isalpha( int code ) throw()
+bool UCS::isalpha( const int code ) throw()
   {
   return ( ( code < 128 && std::isalpha( code ) ) || base_letter( code ) );
   }
 
 
-bool UCS::islower( int code ) throw()
+bool UCS::isdigit( const int code ) throw()
   {
-  if( code < 128 && std::islower( code ) ) return true;
-  code = base_letter( code );
-  return ( code && std::islower( code ) );
+  return ( code <= '9' && code >= '0' );
   }
 
 
-bool UCS::islower_small( int code ) throw()
+bool UCS::islower( const int code ) throw()
+  {
+  if( code < 128 && std::islower( code ) ) return true;
+  const int base = base_letter( code );
+  return ( base && std::islower( base ) );
+  }
+
+
+bool UCS::islower_ambiguous( const int code ) throw()
+  {
+  if( islower_small_ambiguous( code ) ) return true;
+  switch( code )
+    {
+    case 'k': case 'p': case SCCEDI:
+    case SIGRAVE: case SIACUTE: case SICIRCU: case SIDIAER:
+    case SOGRAVE: case SOACUTE: case SOCIRCU: case SOTILDE: case SODIAER:
+    case SUGRAVE: case SUACUTE: case SUCIRCU: case SUDIAER:
+    case  SSCEDI: case SSCARON: case SZCARON:
+      return true;
+    default : return false;
+    }
+  }
+
+
+bool UCS::islower_small( const int code ) throw()
   {
   if( code >= 128 || !std::islower( code ) ) return false;
   switch( code )
@@ -185,29 +209,29 @@ bool UCS::islower_small( int code ) throw()
   }
 
 
-bool UCS::islower_small_ambiguous( int code ) throw()
+bool UCS::islower_small_ambiguous( const int code ) throw()
   {
   if( code >= 128 || !std::islower( code ) ) return false;
   switch( code )
     {
-    case 'c': case 'o': case 's': case 'u': case 'v': case 'w': case 'x':
-    case 'z': return true;
+    case 'c': case 'o': case 's': case 'u': case 'v': case 'w':
+    case 'x': case 'z': return true;
     default : return false;
     }
   }
 
 
-bool UCS::isspace( int code ) throw()
+bool UCS::isspace( const int code ) throw()
   {
   return ( code < 128 && std::isspace( code ) );
   }
 
 
-bool UCS::isupper( int code ) throw()
+bool UCS::isupper( const int code ) throw()
   {
   if( code < 128 && std::isupper( code ) ) return true;
-  code = base_letter( code );
-  return ( code && std::isupper( code ) );
+  const int base = base_letter( code );
+  return ( base && std::isupper( base ) );
   }
 
 
@@ -220,8 +244,9 @@ bool UCS::isvowel( int code ) throw()
   }
 
 
-char UCS::map_to_byte( int code ) throw()
+char UCS::map_to_byte( const int code ) throw()
   {
+  if( code < 0 ) return 0;
   if( code < 256 ) return code;
   switch( code )
     {
@@ -241,7 +266,7 @@ char UCS::map_to_byte( int code ) throw()
   }
 
 
-char * UCS::ucs_to_utf8( int code ) throw()
+char * UCS::ucs_to_utf8( const int code ) throw()
   {
   static char s[7];
 
@@ -264,7 +289,56 @@ char * UCS::ucs_to_utf8( int code ) throw()
   }
 
 
-int UCS::toupper( int code ) throw()
+int UCS::to_nearest_digit( const int code ) throw()
+  {
+  switch( code )
+    {
+    case 'O':
+    case 'Q':
+    case 'o':     return '0';
+    case '|':
+    case 'I':
+    case 'L':
+    case 'l':
+    case SINODOT: return '1';
+    case 'Z':
+    case 'z':     return '2';
+    case 'A':
+    case 'q':     return '4';
+    case 'S':
+    case 's':     return '5';
+    case 'G':
+    case 'b':
+    case SOACUTE: return '6';
+    case 'J':
+    case 'T':     return '7';
+    case '&':
+    case 'B':     return '8';
+    case 'g':     return '9';
+    default:      return code;
+    }
+  }
+
+
+int UCS::to_nearest_letter( const int code ) throw()
+  {
+  switch( code )
+    {
+    case '0': return 'O';
+    case '1': return 'l';
+    case '2': return 'Z';
+    case '4': return 'q';
+    case '5': return 'S';
+    case '6': return SOACUTE;
+    case '7': return 'I';
+    case '8': return 'B';
+    case '9': return 'g';
+    default:  return code;
+    }
+  }
+
+
+int UCS::toupper( const int code ) throw()
   {
   if( code < 128 ) return std::toupper( code );
   switch( code )
@@ -297,6 +371,6 @@ int UCS::toupper( int code ) throw()
     case SUCIRCU: return CUCIRCU;
     case SUDIAER: return CUDIAER;
     case SYACUTE: return CYACUTE;
-    default     : return code;
+    default:      return code;
     }
   }
